@@ -19,24 +19,35 @@
 
 package org.apache.hive.hcatalog.mapreduce;
 
+import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import java.io.IOException;
 import java.net.URI;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
-
-import org.apache.commons.lang.StringUtils;
+import java.util.Queue;
+import java.util.concurrent.Callable;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
+import org.apache.commons.lang3.tuple.Pair;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.fs.permission.FsPermission;
+import org.apache.hadoop.fs.PathFilter;
 import org.apache.hadoop.hive.common.FileUtils;
 import org.apache.hadoop.hive.common.StatsSetupConst;
 import org.apache.hadoop.hive.conf.HiveConf;
+import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
+import org.apache.hadoop.hive.io.HdfsUtils;
 import org.apache.hadoop.hive.metastore.IMetaStoreClient;
 import org.apache.hadoop.hive.metastore.Warehouse;
 import org.apache.hadoop.hive.metastore.api.FieldSchema;
@@ -225,6 +236,15 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
     }
   }
 
+  public static final PathFilter HIDDEN_FILES_PATH_FILTER = new PathFilter() {
+    @Override
+    public boolean accept(Path p) {
+      String name = p.getName();
+      boolean filtered = name.equals(TEMP_DIR_NAME) || name.equals(LOGS_DIR_NAME) || name.equals(SUCCEEDED_FILE_NAME);
+      return !filtered;
+    }
+  };
+
   public static final String SUCCEEDED_FILE_NAME = "_SUCCESS";
   static final String SUCCESSFUL_JOB_OUTPUT_DIR_MARKER =
     "mapreduce.fileoutputcommitter.marksuccessfuljobs";
@@ -323,18 +343,18 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
    * @param params The parameters to store inside the partition
    * @param table The Table metadata object under which this Partition will reside
    * @param fs FileSystem object to operate on the underlying filesystem
-   * @param grpName Group name that owns the table dir
-   * @param perms FsPermission that's the default permission of the table dir.
+   * @param conf HiveConf used to access FS
+   * @param status Permission that's the default permission of the table dir.
    * @return Constructed Partition metadata object
    * @throws java.io.IOException
    */
 
   private Partition constructPartition(
-    JobContext context, OutputJobInfo jobInfo,
-    String partLocnRoot, String dynPartPath, Map<String, String> partKVs,
-    HCatSchema outputSchema, Map<String, String> params,
-    Table table, FileSystem fs,
-    String grpName, FsPermission perms) throws IOException {
+      JobContext context, OutputJobInfo jobInfo,
+      String partLocnRoot, String dynPartPath, Map<String, String> partKVs,
+      HCatSchema outputSchema, Map<String, String> params,
+      Table table, FileSystem fs, HiveConf conf,
+      HdfsUtils.HadoopFileStatus status) throws IOException {
 
     Partition partition = new Partition();
     partition.setDbName(table.getDbName());
@@ -367,22 +387,22 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
       partPath = new Path(finalLocn);
     } else {
       partPath = new Path(partLocnRoot);
+      FileSystem partFs = partPath.getFileSystem(context.getConfiguration());
       int i = 0;
       for (FieldSchema partKey : table.getPartitionKeys()) {
         if (i++ != 0) {
-          fs.mkdirs(partPath); // Attempt to make the path in case it does not exist before we check
-          applyGroupAndPerms(fs, partPath, perms, grpName, false);
+          partFs.mkdirs(partPath); // Attempt to make the path in case it does not exist before we check
+          HdfsUtils.setFullFileStatus(conf, status, status.getFileStatus().getGroup(), fs,
+              partPath, false);
         }
         partPath = constructPartialPartPath(partPath, partKey.getName().toLowerCase(), partKVs);
       }
     }
 
-    // Apply the group and permissions to the leaf partition and files.
-    // Need not bother in case of HDFS as permission is taken care of by setting UMask
-    fs.mkdirs(partPath); // Attempt to make the path in case it does not exist before we check
-    if (!ShimLoader.getHadoopShims().getHCatShim().isFileInHDFS(fs, partPath)) {
-      applyGroupAndPerms(fs, partPath, perms, grpName, true);
-    }
+    // Do not need to set the status on the partition directory. We will do it later recursively.
+    // See: end of the registerPartitions method
+    FileSystem partFs = partPath.getFileSystem(context.getConfiguration());
+    partFs.mkdirs(partPath);
 
     // Set the location in the StorageDescriptor
     if (dynamicPartitioningUsed) {
@@ -400,26 +420,6 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
     return partition;
   }
 
-  private void applyGroupAndPerms(FileSystem fs, Path dir, FsPermission permission,
-                  String group, boolean recursive)
-    throws IOException {
-    if(LOG.isDebugEnabled()) {
-      LOG.debug("applyGroupAndPerms : " + dir +
-          " perms: " + permission +
-          " group: " + group + " recursive: " + recursive);
-    }
-    fs.setPermission(dir, permission);
-    if (recursive) {
-      for (FileStatus fileStatus : fs.listStatus(dir)) {
-        if (fileStatus.isDir()) {
-          applyGroupAndPerms(fs, fileStatus.getPath(), permission, group, true);
-        } else {
-          fs.setPermission(fileStatus.getPath(), permission);
-        }
-      }
-    }
-  }
-
   private String getFinalDynamicPartitionDestination(Table table, Map<String, String> partKVs,
       OutputJobInfo jobInfo) {
     Path partPath = new Path(table.getTTable().getSd().getLocation());
@@ -488,139 +488,148 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
   }
 
   /**
-   * Move all of the files from the temp directory to the final location
-   * @param fs the output file system
-   * @param file the file to move
+   * Move task output from the temp directory to the final location
+   * @param srcf the file to move
    * @param srcDir the source directory
    * @param destDir the target directory
-   * @param dryRun - a flag that simply tests if this move would succeed or not based
-   *                 on whether other files exist where we're trying to copy
+   * @param immutable - whether table is immutable.
    * @throws java.io.IOException
    */
-  private void moveTaskOutputs(FileSystem fs, Path file, Path srcDir,
-                 Path destDir, final boolean dryRun, boolean immutable
-      ) throws IOException {
-    if(LOG.isDebugEnabled()) {
+  private void moveTaskOutputs(final Configuration conf, Path srcf, Path srcDir,
+      Path destDir, boolean immutable) throws IOException {
+    if (LOG.isDebugEnabled()) {
       LOG.debug("moveTaskOutputs "
-          + file + " from: " + srcDir + " to: " + destDir
-          + " dry: " + dryRun + " immutable: " + immutable);
+          + srcf + " from: " + srcDir + " to: " + destDir + " immutable: " + immutable);
     }
 
     if (dynamicPartitioningUsed) {
       immutable = true; // Making sure we treat dynamic partitioning jobs as if they were immutable.
     }
 
-    if (file.getName().equals(TEMP_DIR_NAME) || file.getName().equals(LOGS_DIR_NAME) || file.getName().equals(SUCCEEDED_FILE_NAME)) {
+    final FileSystem srcFs = srcf.getFileSystem(conf);
+    final FileSystem destFs = destDir.getFileSystem(conf);
+    final boolean canRename = srcFs.getUri().equals(destFs.getUri());
+
+    if (destFs.exists(destDir) && !destFs.getFileStatus(destDir).isDirectory()) {
+      throw new HCatException(ErrorType.ERROR_MOVE_FAILED, "Destination is not directory " + destDir);
+    }
+
+    LinkedList<Pair<Path, Path>> moves = new LinkedList<>();
+    if (customDynamicLocationUsed) {
+      if (immutable && destFs.exists(destDir)
+          && !org.apache.hadoop.hive.metastore.utils.FileUtils.isDirEmpty(destFs, destDir)) {
+        throw new HCatException(ErrorType.ERROR_DUPLICATE_PARTITION,
+            "Data already exists in " + destDir
+                + ", duplicate publish not possible.");
+      }
+      moves.add(Pair.of(srcf, destDir));
+    } else {
+      Queue<FileStatus> srcQ = new LinkedList<>();
+      FileStatus[] contents = srcFs.listStatus(srcf, HIDDEN_FILES_PATH_FILTER);
+      if (contents.length == 0) {
+        // nothing to move
+        return;
+      }
+      Collections.addAll(srcQ, contents);
+
+      while (!srcQ.isEmpty()) {
+        FileStatus srcStatus = srcQ.remove();
+        Path srcF = srcStatus.getPath();
+        final Path finalOutputPath = getFinalPath(destFs, srcF, srcDir, destDir, immutable);
+        if (immutable && destFs.exists(finalOutputPath)
+            && !org.apache.hadoop.hive.metastore.utils.FileUtils.isDirEmpty(destFs, finalOutputPath)) {
+          if (partitionsDiscoveredByPath.containsKey(srcF.toString())) {
+            throw new HCatException(ErrorType.ERROR_DUPLICATE_PARTITION,
+                "Data already exists in " + finalOutputPath + ", duplicate publish not possible.");
+          }
+          // parent directory may exist for multi-partitions, check lower level partitions
+          Collections.addAll(srcQ, srcFs.listStatus(srcF, HIDDEN_FILES_PATH_FILTER));
+        } else if (srcStatus.isDirectory()) {
+          if (canRename && dynamicPartitioningUsed) {
+            // If it is partition, move the partition directory instead of each file.
+            moves.add(Pair.of(srcF, finalOutputPath));
+          } else {
+            Collections.addAll(srcQ, srcFs.listStatus(srcF, HIDDEN_FILES_PATH_FILTER));
+          }
+        } else {
+          moves.add(Pair.of(srcF, finalOutputPath));
+        }
+      }
+    }
+
+    bulkMoveFiles(conf, srcFs, destFs, moves);
+  }
+
+  /**
+   * Bulk move files from source to destination.
+   * @param srcFs the source filesystem where the source files are
+   * @param destFs the destionation filesystem where the destionation files are
+   * @param pairs list of pairs of <source_path, destination_path>, move source_path to destination_path
+   * @throws java.io.IOException
+   */
+  private void bulkMoveFiles(final Configuration conf, final FileSystem srcFs, final FileSystem destFs,
+      final List<Pair<Path, Path>> pairs) throws IOException {
+    if (pairs.isEmpty()) {
+      return;
+    }
+    final boolean canRename = srcFs.getUri().equals(destFs.getUri());
+    final List<Future<Pair<Path, Path>>> futures = new LinkedList<>();
+    final int moveThreadsCount = conf.getInt(ConfVars.HIVE_MOVE_FILES_THREAD_COUNT.varname, 25);
+
+    if (moveThreadsCount <= 0) {
+      for (final Pair<Path, Path> pair: pairs) {
+        Path srcP = pair.getLeft();
+        Path dstP = pair.getRight();
+        if (!moveFile(srcFs, srcP, destFs, dstP, conf, canRename)) {
+          throw new HCatException(ErrorType.ERROR_MOVE_FAILED,
+              "Unable to move from " + srcP + " to " + dstP);
+        }
+      }
       return;
     }
 
-    final Path finalOutputPath = getFinalPath(fs, file, srcDir, destDir, immutable);
-    FileStatus fileStatus = fs.getFileStatus(file);
+    final ExecutorService pool = Executors.newFixedThreadPool(moveThreadsCount,
+        new ThreadFactoryBuilder().setDaemon(true).setNameFormat("Move-Thread-%d").build());
 
-    if (!fileStatus.isDir()) {
-      if (dryRun){
-        if (immutable){
-          // Dryrun checks are meaningless for mutable table - we should always succeed
-          // unless there is a runtime IOException.
-          if(LOG.isDebugEnabled()) {
-            LOG.debug("Testing if moving file: [" + file + "] to ["
-                + finalOutputPath + "] would cause a problem");
-          }
-          if (fs.exists(finalOutputPath)) {
-            throw new HCatException(ErrorType.ERROR_MOVE_FAILED, "Data already exists in "
-                + finalOutputPath + ", duplicate publish not possible.");
-          }
-        }
-      } else {
-        if(LOG.isDebugEnabled()) {
-          LOG.debug("Moving file: [ " + file + "] to [" + finalOutputPath + "]");
-        }
-        // Make sure the parent directory exists.  It is not an error
-        // to recreate an existing directory
-        fs.mkdirs(finalOutputPath.getParent());
-        if (!fs.rename(file, finalOutputPath)) {
-          if (!fs.delete(finalOutputPath, true)) {
-            throw new HCatException(ErrorType.ERROR_MOVE_FAILED, "Failed to delete existing path " + finalOutputPath);
-          }
-          if (!fs.rename(file, finalOutputPath)) {
-            throw new HCatException(ErrorType.ERROR_MOVE_FAILED, "Failed to move output to " + finalOutputPath);
-          }
-        }
-      }
-    } else {
-
-      FileStatus[] children = fs.listStatus(file);
-      FileStatus firstChild = null;
-      if (children != null) {
-        int index=0;
-        while (index < children.length) {
-          if ( !children[index].getPath().getName().equals(TEMP_DIR_NAME)
-              && !children[index].getPath().getName().equals(LOGS_DIR_NAME)
-              && !children[index].getPath().getName().equals(SUCCEEDED_FILE_NAME)) {
-            firstChild = children[index];
-            break;
-          }
-          index++;
-        }
-      }
-      if(firstChild!=null && firstChild.isDir()) {
-        // If the first child is directory, then rest would be directory too according to HCatalog dir structure
-        // recurse in that case
-        for (FileStatus child : children) {
-          moveTaskOutputs(fs, child.getPath(), srcDir, destDir, dryRun, immutable);
-        }
-      } else {
-
-        if (!dryRun) {
-          if (dynamicPartitioningUsed) {
-
-            // Optimization: if the first child is file, we have reached the leaf directory, move the parent directory itself
-            // instead of moving each file under the directory. See HCATALOG-538
-            // Note for future Append implementation : This optimization is another reason dynamic
-            // partitioning is currently incompatible with append on mutable tables.
-
-            final Path parentDir = finalOutputPath.getParent();
-            // Create the directory
-            Path placeholder = new Path(parentDir, "_placeholder" + String.valueOf(Math.random()));
-            if (fs.mkdirs(parentDir)) {
-              // It is weird but we need a placeholder,
-              // otherwise rename cannot move file to the right place
-              fs.create(placeholder).close();
-            }
-            if (LOG.isDebugEnabled()) {
-              LOG.debug("Moving directory: " + file + " to " + parentDir);
-            }
-
-            // If custom dynamic location provided, need to rename to final output path
-            Path dstPath = !customDynamicLocationUsed ? parentDir : finalOutputPath;
-            if (!fs.rename(file, dstPath)) {
-              final String msg = "Failed to move file: " + file + " to " + dstPath;
-              LOG.error(msg);
-              throw new HCatException(ErrorType.ERROR_MOVE_FAILED, msg);
-            }
-            fs.delete(placeholder, false);
+    for (final Pair<Path, Path> pair: pairs) {
+      Path srcP = pair.getLeft();
+      Path dstP = pair.getRight();
+      futures.add(pool.submit(new Callable<Pair<Path, Path>>() {
+        @Override
+        public Pair<Path, Path> call() throws IOException {
+          if (moveFile(srcFs, srcP, destFs, dstP, conf, canRename)) {
+            return pair;
           } else {
-
-            // In case of no partition we have to move each file
-            for (FileStatus child : children) {
-              moveTaskOutputs(fs, child.getPath(), srcDir, destDir, dryRun, immutable);
-            }
-
+            throw new HCatException(ErrorType.ERROR_MOVE_FAILED,
+                "Unable to move from " + srcP + " to " + dstP);
           }
-
-        } else {
-          if(immutable && fs.exists(finalOutputPath) &&
-              !org.apache.hadoop.hive.metastore.utils.FileUtils.isDirEmpty(fs, finalOutputPath)) {
-
-            throw new HCatException(ErrorType.ERROR_DUPLICATE_PARTITION, "Data already exists in " + finalOutputPath
-                + ", duplicate publish not possible.");
-          }
-
         }
+      }));
+    }
+    pool.shutdown();
+    for (Future<Pair<Path, Path>> future : futures) {
+      try {
+        future.get();
+      } catch (Exception e) {
+        pool.shutdownNow();
+        throw new HCatException(ErrorType.ERROR_MOVE_FAILED, e.getMessage());
       }
     }
   }
 
+  private boolean moveFile(final FileSystem srcFs, final Path srcf, final FileSystem destFs, final Path destf,
+      final Configuration conf, final boolean canRename) throws IOException {
+    LOG.debug("Moving src: {}, to dest: {}", srcf, destf);
+    boolean moved;
+    if (canRename) {
+      destFs.mkdirs(destf.getParent());
+      moved = srcFs.rename(srcf, destf);
+    } else {
+      moved = FileUtil.copy(srcFs, srcf, destFs, destf, true, false, conf);
+    }
+    return moved;
+  }
+
   /**
    * Find the final name of a given output file, given the output directory
    * and the work directory. If immutable, attempt to create file of name
@@ -703,7 +712,7 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
 
       // construct a path pattern (e.g., /*/*) to find all dynamically generated paths
       String dynPathSpec = loadPath.toUri().getPath();
-      dynPathSpec = dynPathSpec.replaceAll("__HIVE_DEFAULT_PARTITION__", "*");
+      dynPathSpec = dynPathSpec.replace("__HIVE_DEFAULT_PARTITION__", "*");
 
       //      LOG.info("Searching for "+dynPathSpec);
       Path pathPattern = new Path(dynPathSpec);
@@ -725,7 +734,7 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
               + "exceeds configured max allowable partitions["
               + maxDynamicPartitions
               + "], increase parameter ["
-              + HiveConf.ConfVars.DYNAMICPARTITIONMAXPARTS.varname
+              + HiveConf.ConfVars.DYNAMIC_PARTITION_MAX_PARTS.varname
               + "] if needed.");
         }
 
@@ -779,7 +788,7 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
         // Move data from temp directory the actual table directory
         // No metastore operation required.
         Path src = new Path(jobInfo.getLocation());
-        moveTaskOutputs(fs, src, src, tblPath, false, table.isImmutable());
+        moveTaskOutputs(conf, src, src, tblPath, table.isImmutable());
         if (!src.equals(tblPath)) {
           fs.delete(src, true);
         }
@@ -794,21 +803,19 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
       StorerInfo storer = InternalUtil.extractStorerInfo(table.getTTable().getSd(),
           table.getParameters());
 
-      FileStatus tblStat = fs.getFileStatus(tblPath);
-      String grpName = tblStat.getGroup();
-      FsPermission perms = tblStat.getPermission();
+      HdfsUtils.HadoopFileStatus status = new HdfsUtils.HadoopFileStatus(conf, fs, tblPath);
 
       List<Partition> partitionsToAdd = new ArrayList<Partition>();
       if (!dynamicPartitioningUsed) {
         partitionsToAdd.add(constructPartition(context, jobInfo, tblPath.toString(), null,
             jobInfo.getPartitionValues(), jobInfo.getOutputSchema(), getStorerParameterMap(storer),
-            table, fs, grpName, perms));
+            table, fs, hiveConf, status));
       } else {
         for (Entry<String, Map<String, String>> entry : partitionsDiscoveredByPath.entrySet()) {
           partitionsToAdd.add(constructPartition(context, jobInfo,
               getPartitionRootLocation(entry.getKey(), entry.getValue().size()), entry.getKey(),
               entry.getValue(), jobInfo.getOutputSchema(), getStorerParameterMap(storer), table,
-              fs, grpName, perms));
+              fs, hiveConf, status));
         }
       }
 
@@ -844,8 +851,7 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
           // check here for each dir we're copying out, to see if it
           // already exists, error out if so.
           // Also, treat dyn-writes as writes to immutable tables.
-          moveTaskOutputs(fs, src, src, tblPath, true, true); // dryRun = true, immutable = true
-          moveTaskOutputs(fs, src, src, tblPath, false, true);
+          moveTaskOutputs(conf, src, src, tblPath, table.isImmutable());
           if (!src.equals(tblPath)){
             fs.delete(src, true);
           }
@@ -885,8 +891,7 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
             Partition p = partitionsToAdd.get(0);
             Path src = new Path(jobInfo.getLocation());
             Path dest = new Path(p.getSd().getLocation());
-            moveTaskOutputs(fs, src, src, dest, true, table.isImmutable());
-            moveTaskOutputs(fs,src,src,dest,false,table.isImmutable());
+            moveTaskOutputs(conf, src, src, dest, table.isImmutable());
             if (!src.equals(dest)){
               if (src.toString().matches(".*" + Path.SEPARATOR + SCRATCH_DIR_NAME + "\\d\\.?\\d+.*")){
                 // src is scratch directory, need to trim the part key value pairs from path
@@ -934,8 +939,7 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
             // Dynamic partitioning usecase
             if (!customDynamicLocationUsed) {
               Path src = new Path(ptnRootLocation);
-              moveTaskOutputs(fs, src, src, tblPath, true, true); // dryRun = true, immutable = true
-              moveTaskOutputs(fs, src, src, tblPath, false, true);
+              moveTaskOutputs(conf, src, src, tblPath, true);
               if (!src.equals(tblPath)){
                 fs.delete(src, true);
               }
@@ -950,9 +954,9 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
         // Set permissions appropriately for each of the partitions we just created
         // so as to have their permissions mimic the table permissions
         for (Partition p : partitionsAdded){
-          applyGroupAndPerms(fs,new Path(p.getSd().getLocation()),tblStat.getPermission(),tblStat.getGroup(),true);
+          HdfsUtils.setFullFileStatus(conf, status, status.getFileStatus().getGroup(), fs,
+              new Path(p.getSd().getLocation()), true);
         }
-
       }
     } catch (Exception e) {
       if (partitionsAdded.size() > 0) {
@@ -987,8 +991,7 @@ class FileOutputCommitterContainer extends OutputCommitterContainer {
     for (Entry<String, Map<String, String>> entry : partitionsDiscoveredByPath.entrySet()) {
       Path src = new Path(entry.getKey());
       Path destPath = new Path(getFinalDynamicPartitionDestination(table, entry.getValue(), jobInfo));
-      moveTaskOutputs(fs, src, src, destPath, true, true); // dryRun = true, immutable = true
-      moveTaskOutputs(fs, src, src, destPath, false, true);
+      moveTaskOutputs(conf, src, src, destPath, true);
     }
     // delete the parent temp directory of all custom dynamic partitions
     Path parentPath = new Path(getCustomPartitionRootLocation(jobInfo, conf));
