@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -21,20 +21,17 @@ package org.apache.hadoop.hive.ql.exec;
 import java.io.IOException;
 import java.io.Serializable;
 import java.util.ArrayList;
-import java.util.Collection;
-import java.util.HashSet;
 import java.util.List;
-import java.util.concurrent.Callable;
 import java.util.concurrent.Future;
 
-import com.esotericsoftware.kryo.KryoException;
 import org.apache.commons.lang3.tuple.ImmutablePair;
 import org.apache.commons.lang3.tuple.Pair;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.commons.lang3.ArrayUtils;
 import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.hive.common.ObjectPair;
+import org.apache.hadoop.hive.conf.Constants;
 import org.apache.hadoop.hive.conf.HiveConf;
+import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
+import org.apache.hadoop.hive.ql.CompilationOpContext;
 import org.apache.hadoop.hive.ql.HashTableLoaderFactory;
 import org.apache.hadoop.hive.ql.exec.mr.ExecMapperContext;
 import org.apache.hadoop.hive.ql.exec.persistence.BytesBytesMultiHashMap;
@@ -43,27 +40,45 @@ import org.apache.hadoop.hive.ql.exec.persistence.HybridHashTableContainer.HashP
 import org.apache.hadoop.hive.ql.exec.persistence.KeyValueContainer;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinBytesTableContainer;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinBytesTableContainer.KeyValueHelper;
+import org.apache.hadoop.hive.ql.exec.persistence.AbstractRowContainer;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinKey;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinObjectSerDeContext;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinRowContainer;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinTableContainer;
+import org.apache.hadoop.hive.ql.exec.persistence.ReusableGetAdaptorDirectAccess;
+import org.apache.hadoop.hive.ql.exec.persistence.MapJoinTableContainer.NonMatchedSmallTableIterator;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinTableContainer.ReusableGetAdaptor;
 import org.apache.hadoop.hive.ql.exec.persistence.MapJoinTableContainerSerDe;
+import org.apache.hadoop.hive.ql.exec.persistence.MatchTracker;
 import org.apache.hadoop.hive.ql.exec.persistence.ObjectContainer;
 import org.apache.hadoop.hive.ql.exec.persistence.UnwrapRowContainer;
+import org.apache.hadoop.hive.ql.exec.tez.LlapObjectCache;
+import org.apache.hadoop.hive.ql.exec.tez.LlapObjectSubCache;
 import org.apache.hadoop.hive.ql.io.HiveKey;
 import org.apache.hadoop.hive.ql.log.PerfLogger;
 import org.apache.hadoop.hive.ql.metadata.HiveException;
+import org.apache.hadoop.hive.ql.plan.JoinCondDesc;
+import org.apache.hadoop.hive.ql.plan.JoinDesc;
 import org.apache.hadoop.hive.ql.plan.MapJoinDesc;
 import org.apache.hadoop.hive.ql.plan.TableDesc;
 import org.apache.hadoop.hive.ql.plan.api.OperatorType;
-import org.apache.hadoop.hive.serde2.SerDe;
+import org.apache.hadoop.hive.ql.session.SessionState;
+import org.apache.hadoop.hive.serde2.AbstractSerDe;
 import org.apache.hadoop.hive.serde2.SerDeException;
-import org.apache.hadoop.hive.serde2.SerDeUtils;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspector;
+import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspectorConverters;
+import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspectorUtils;
+import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspectorConverters.Converter;
+import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspectorUtils.ObjectInspectorCopyOption;
 import org.apache.hadoop.io.BytesWritable;
 import org.apache.hadoop.io.Writable;
 import org.apache.hive.common.util.ReflectionUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import com.google.common.annotations.VisibleForTesting;
+import com.esotericsoftware.kryo.KryoException;
+import com.google.common.base.Preconditions;
 
 /**
  * Map side Join operator implementation.
@@ -71,15 +86,14 @@ import org.apache.hive.common.util.ReflectionUtil;
 public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implements Serializable {
 
   private static final long serialVersionUID = 1L;
-  private static final Log LOG = LogFactory.getLog(MapJoinOperator.class.getName());
+  private static final Logger LOG = LoggerFactory.getLogger(MapJoinOperator.class.getName());
   private static final String CLASS_NAME = MapJoinOperator.class.getName();
-  private final PerfLogger perfLogger = PerfLogger.getPerfLogger();
+  private transient final PerfLogger perfLogger = SessionState.getPerfLogger();
 
   private transient String cacheKey;
   private transient ObjectCache cache;
 
   protected HashTableLoader loader;
-  private boolean loadCalled;
 
   protected transient MapJoinTableContainer[] mapJoinTables;
   private transient MapJoinTableContainerSerDe[] mapJoinTableSerdes;
@@ -94,7 +108,35 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
   protected HybridHashTableContainer firstSmallTable; // The first small table;
                                                       // Only this table has spilled big table rows
 
-  public MapJoinOperator() {
+  /*
+   * FULL OUTER MapJoin members.
+   */
+  protected transient boolean isFullOuterMapJoin;     // Are we doing a FULL OUTER MapJoin?
+
+  protected transient int fullOuterBigTableRetainSize;
+                                                      // The number of Big Table columns being
+                                                      // retained in the output result for
+                                                      // FULL OUTER MapJoin.
+
+  /*
+   * Small Table key match tracking used for FULL OUTER MapJoin.  Otherwise, null.
+   * Since the Small Table hash table can be shared among vertces, we require this non-shared object
+   * for our vertex (i.e. operator private) key match tracking.
+   */
+  protected transient MatchTracker matchTracker;
+
+  protected transient boolean isTestingNoHashTableLoad;
+  // Only used in bucket map join.
+  private transient int numBuckets = -1;
+  private transient int bucketId = -1;
+
+  /** Kryo ctor. */
+  protected MapJoinOperator() {
+    super();
+  }
+
+  public MapJoinOperator(CompilationOpContext ctx) {
+    super(ctx);
   }
 
   public MapJoinOperator(AbstractMapJoinOperator<? extends MapJoinDesc> mjop) {
@@ -120,38 +162,63 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     return HashTableLoaderFactory.getLoader(hconf);
   }
 
+  public String getCacheKey() {
+    return cacheKey;
+  }
+
   @Override
-  protected Collection<Future<?>> initializeOp(Configuration hconf) throws HiveException {
+  protected void initializeOp(Configuration hconf) throws HiveException {
     this.hconf = hconf;
     unwrapContainer = new UnwrapRowContainer[conf.getTagLength()];
 
-    Collection<Future<?>> result = super.initializeOp(hconf);
-    if (result == null) {
-      result = new HashSet<Future<?>>();
-    }
+    super.initializeOp(hconf);
 
     int tagLen = conf.getTagLength();
 
     // On Tez only: The hash map might already be cached in the container we run
     // the task in. On MR: The cache is a no-op.
-    cacheKey = HiveConf.getVar(hconf, HiveConf.ConfVars.HIVEQUERYID)
-      + "__HASH_MAP_"+this.getOperatorId()+"_container";
-
-    cache = ObjectCacheFactory.getCache(hconf);
+    String queryId = HiveConf.getVar(hconf, HiveConf.ConfVars.HIVE_QUERY_ID);
+    // The cacheKey may have already been defined in the MapJoin conf spec
+    // as part of the Shared Work Optimization if it can be reused among
+    // multiple mapjoin operators. In that case, we take that key from conf
+    // and append this.getClass().getName() to disambiguate between different
+    // classes that may be using the same source data, e.g.
+    // VectorMapJoinInnerGenerateResultOperator and VectorMapJoinLeftSemiLongOperator.
+    // If the cacheKey is not defined in the conf, then we generate it.
+    cacheKey = conf.getCacheKey() == null ?
+        MapJoinDesc.generateCacheKey(this.getOperatorId()) :
+        conf.getCacheKey() + "_" + this.getClass().getName();
+    cache = ObjectCacheFactory.getCache(hconf, queryId, false);
     loader = getHashTableLoader(hconf);
 
+    bucketId = hconf.getInt(Constants.LLAP_BUCKET_ID, -1);
+    numBuckets = hconf.getInt(Constants.LLAP_NUM_BUCKETS, -1);
+
     hashMapRowGetters = null;
 
     mapJoinTables = new MapJoinTableContainer[tagLen];
     mapJoinTableSerdes = new MapJoinTableContainerSerDe[tagLen];
     hashTblInitedOnce = false;
 
+    // Reset grace hashjoin context so that there is no state maintained when operator/work is
+    // retrieved from object cache
+    hybridMapJoinLeftover = false;
+    firstSmallTable = null;
+
+    doFullOuterMapJoinInit();
+
     generateMapMetaData();
 
+    isTestingNoHashTableLoad = HiveConf.getBoolVar(hconf,
+        HiveConf.ConfVars.HIVE_MAPJOIN_TESTING_NO_HASH_TABLE_LOAD);
+    if (isTestingNoHashTableLoad) {
+      return;
+    }
+
     final ExecMapperContext mapContext = getExecContext();
     final MapredContext mrContext = MapredContext.get();
 
-    if (!conf.isBucketMapJoin()) {
+    if (!conf.isBucketMapJoin() && !conf.isDynamicPartitionHashJoin()) {
       /*
        * The issue with caching in case of bucket map join is that different tasks
        * process different buckets and if the container is reused to join a different bucket,
@@ -160,32 +227,21 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
        * requires changes in the Tez API with regard to finding bucket id and
        * also ability to schedule tasks to re-use containers that have cached the specific bucket.
        */
-      if (isLogInfoEnabled) {
-        LOG.info("This is not bucket map join, so cache");
-      }
+      LOG.debug("This is not bucket map join, so cache");
 
       Future<Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]>> future =
           cache.retrieveAsync(
-              cacheKey,
-              new Callable<Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]>>() {
-                @Override
-                public Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> call()
-                    throws HiveException {
-                  return loadHashTable(mapContext, mrContext);
-                }
-              });
-      result.add(future);
-    } else if (mapContext == null || mapContext.getLocalWork() == null
-        || mapContext.getLocalWork().getInputFileChangeSensitive() == false) {
+              cacheKey, () ->loadHashTable(mapContext, mrContext));
+      asyncInitOperations.add(future);
+    } else if (!isInputFileChangeSensitive(mapContext)) {
       loadHashTable(mapContext, mrContext);
       hashTblInitedOnce = true;
     }
-    return result;
   }
 
   @SuppressWarnings("unchecked")
   @Override
-  protected final void completeInitializationOp(Object[] os) throws HiveException {
+  protected void completeInitializationOp(Object[] os) throws HiveException {
     if (os.length != 0) {
       Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> pair =
           (Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]>) os[0];
@@ -197,10 +253,18 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
         }
       }
 
-      if (!loadCalled && spilled) {
+      if (spilled) {
         // we can't use the cached table because it has spilled.
+
         loadHashTable(getExecContext(), MapredContext.get());
       } else {
+        if (LOG.isDebugEnabled()) {
+          String s = "Using tables from cache: [";
+          for (MapJoinTableContainer c : pair.getLeft()) {
+            s += ((c == null) ? "null" : c.getClass().getSimpleName()) + ", ";
+          }
+          LOG.debug(s + "]");
+        }
         // let's use the table from the cache.
         mapJoinTables = pair.getLeft();
         mapJoinTableSerdes = pair.getRight();
@@ -216,6 +280,32 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     }
   }
 
+  /*
+   * Do initialization for FULL OUTER MapJoin.
+   *
+   * Currently, we do not support FULL OUTER MapJoin for N-way.
+   */
+  private void doFullOuterMapJoinInit() {
+
+    // This will be set during the first process call or during closeOp if no rows processed.
+    matchTracker = null;
+
+    isFullOuterMapJoin = (condn.length == 1 && condn[0].getType() == JoinDesc.FULL_OUTER_JOIN);
+    if (isFullOuterMapJoin) {
+      fullOuterBigTableRetainSize = conf.getRetainList().get(posBigTable).size();
+    } else {
+      fullOuterBigTableRetainSize = 0;
+    }
+  }
+
+  @VisibleForTesting
+  public void setTestMapJoinTableContainer(int posSmallTable,
+      MapJoinTableContainer testMapJoinTableContainer,
+      MapJoinTableContainerSerDe mapJoinTableContainerSerDe) {
+    mapJoinTables[posSmallTable] = testMapJoinTableContainer;
+    mapJoinTableSerdes[posSmallTable] = mapJoinTableContainerSerDe;
+  }
+
   @Override
   protected List<ObjectInspector> getValueObjectInspectors(
       byte alias, List<ObjectInspector>[] aliasToObjectInspectors) {
@@ -223,19 +313,30 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     if (valueIndex == null) {
       return super.getValueObjectInspectors(alias, aliasToObjectInspectors);
     }
-    unwrapContainer[alias] = new UnwrapRowContainer(alias, valueIndex, hasFilter(alias));
 
     List<ObjectInspector> inspectors = aliasToObjectInspectors[alias];
-
     int bigPos = conf.getPosBigTable();
+    Converter[] converters = new Converter[valueIndex.length];
     List<ObjectInspector> valueOI = new ArrayList<ObjectInspector>();
     for (int i = 0; i < valueIndex.length; i++) {
       if (valueIndex[i] >= 0 && !joinKeysObjectInspectors[bigPos].isEmpty()) {
-        valueOI.add(joinKeysObjectInspectors[bigPos].get(valueIndex[i]));
+        if (conf.getNoOuterJoin()) {
+          valueOI.add(joinKeysObjectInspectors[bigPos].get(valueIndex[i]));
+        } else {
+          // It is an outer join. We are going to add the inspector from the
+          // inner side, but the key value will come from the outer side, so
+          // we need to create a converter from inputOI to outputOI.
+          valueOI.add(inspectors.get(i));
+          converters[i] = ObjectInspectorConverters.getConverter(
+                  joinKeysObjectInspectors[bigPos].get(valueIndex[i]), inspectors.get(i));
+        }
       } else {
         valueOI.add(inspectors.get(i));
       }
     }
+
+    unwrapContainer[alias] = new UnwrapRowContainer(alias, valueIndex, converters, hasFilter(alias));
+
     return valueOI;
   }
 
@@ -245,10 +346,10 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
 
     try {
       TableDesc keyTableDesc = conf.getKeyTblDesc();
-      SerDe keySerializer = (SerDe) ReflectionUtil.newInstance(
-          keyTableDesc.getDeserializerClass(), null);
-      SerDeUtils.initializeSerDe(keySerializer, null, keyTableDesc.getProperties(), null);
-      MapJoinObjectSerDeContext keyContext = new MapJoinObjectSerDeContext(keySerializer, false);
+      AbstractSerDe keySerDe = (AbstractSerDe) ReflectionUtil.newInstance(
+          keyTableDesc.getSerDeClass(), null);
+      keySerDe.initialize(null, keyTableDesc.getProperties(), null);
+      MapJoinObjectSerDeContext keyContext = new MapJoinObjectSerDeContext(keySerDe, false);
       for (int pos = 0; pos < order.length; pos++) {
         if (pos == posBigTable) {
           continue;
@@ -259,9 +360,9 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
         } else {
           valueTableDesc = conf.getValueFilteredTblDescs().get(pos);
         }
-        SerDe valueSerDe = (SerDe) ReflectionUtil.newInstance(
-            valueTableDesc.getDeserializerClass(), null);
-        SerDeUtils.initializeSerDe(valueSerDe, null, valueTableDesc.getProperties(), null);
+        AbstractSerDe valueSerDe = (AbstractSerDe) ReflectionUtil.newInstance(
+            valueTableDesc.getSerDeClass(), null);
+        valueSerDe.initialize(null, valueTableDesc.getProperties(), null);
         MapJoinObjectSerDeContext valueContext =
             new MapJoinObjectSerDeContext(valueSerDe, hasFilter(pos));
         mapJoinTableSerdes[pos] = new MapJoinTableContainerSerDe(keyContext, valueContext);
@@ -271,26 +372,15 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     }
   }
 
-  protected Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> loadHashTable(
-      ExecMapperContext mapContext, MapredContext mrContext) throws HiveException {
-    loadCalled = true;
-
-    if (this.hashTblInitedOnce
-        && ((mapContext == null) || (mapContext.getLocalWork() == null) || (mapContext
-            .getLocalWork().getInputFileChangeSensitive() == false))) {
-      // no need to reload
-      return new ImmutablePair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]>(
-          mapJoinTables, mapJoinTableSerdes);
-    }
-
-    perfLogger.PerfLogBegin(CLASS_NAME, PerfLogger.LOAD_HASHTABLE);
+  // Core logic to load hash table using HashTableLoader
+  private Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> loadHashTableInternal(
+          ExecMapperContext mapContext, MapredContext mrContext) throws HiveException {
+    perfLogger.perfLogBegin(CLASS_NAME, PerfLogger.LOAD_HASHTABLE);
     loader.init(mapContext, mrContext, hconf, this);
     try {
       loader.load(mapJoinTables, mapJoinTableSerdes);
     } catch (HiveException e) {
-      if (isLogInfoEnabled) {
-        LOG.info("Exception loading hash tables. Clearing partially loaded hash table containers.");
-      }
+      LOG.info("Exception loading hash tables. Clearing partially loaded hash table containers");
 
       // there could be some spilled partitions which needs to be cleaned up
       clearAllTableContainers();
@@ -299,15 +389,65 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
 
     hashTblInitedOnce = true;
 
-    Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> pair
-      = new ImmutablePair<MapJoinTableContainer[],
-      MapJoinTableContainerSerDe[]> (mapJoinTables, mapJoinTableSerdes);
+    Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> pair =
+            new ImmutablePair<> (mapJoinTables, mapJoinTableSerdes);
 
-    perfLogger.PerfLogEnd(CLASS_NAME, PerfLogger.LOAD_HASHTABLE);
+    perfLogger.perfLogEnd(CLASS_NAME, PerfLogger.LOAD_HASHTABLE);
 
+    if (canSkipJoinProcessing(mapContext)) {
+      LOG.info("Skipping big table join processing for " + this.toString());
+      this.setDone(true);
+    }
     return pair;
   }
 
+  // Load Hash table for Bucket MapJoin
+  private Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> loadHashTableBMJ(
+          ExecMapperContext mapContext, MapredContext mrContext) throws HiveException {
+    // Bucket MapJoin in LLAP, make sure the caches are populated.
+    // Get the subcache.
+
+    LlapObjectSubCache<Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]>> subCache =
+            new LlapObjectSubCache<>(cache, cacheKey + "_BMJ", numBuckets);
+
+    subCache.lock(bucketId);
+    try {
+      Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> pair =
+              subCache.get(bucketId);
+      if (pair != null) {
+        // match found! use it
+        // update the tables.
+        mapJoinTables = pair.getLeft();
+        mapJoinTableSerdes = pair.getRight();
+        return pair;
+      }
+      pair = loadHashTableInternal(mapContext, mrContext);
+
+      // update the subcache
+      subCache.set(pair, bucketId);
+      return pair;
+    } finally {
+      subCache.unlock(bucketId);
+    }
+  }
+
+  protected Pair<MapJoinTableContainer[], MapJoinTableContainerSerDe[]> loadHashTable(
+      ExecMapperContext mapContext, MapredContext mrContext) throws HiveException {
+    if (canSkipReload(mapContext)) {
+      // no need to reload
+      return new ImmutablePair<>(mapJoinTables, mapJoinTableSerdes);
+    }
+
+    if (conf.isBucketMapJoin() && cache instanceof LlapObjectCache &&
+            numBuckets > 0 && HiveConf.getBoolVar(hconf,
+            ConfVars.HIVE_TEZ_BMJ_USE_SUBCACHE)) {
+      // Bucket MapJoin in LLAP
+      return loadHashTableBMJ(mapContext, mrContext);
+    }
+
+    return loadHashTableInternal(mapContext, mrContext);
+  }
+
   // Load the hash table
   @Override
   public void cleanUpInputFileChangedOp() throws HiveException {
@@ -319,6 +459,16 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     return dest.setFromRow(row, joinKeys[alias], joinKeysObjectInspectors[alias]);
   }
 
+  /*
+   * This variation is for FULL OUTER MapJoin.  It does key match tracking only if the key has
+   * no NULLs.
+   */
+  protected JoinUtil.JoinResult setMapJoinKeyNoNulls(
+      ReusableGetAdaptor dest, Object row, byte alias, MatchTracker matchTracker)
+          throws HiveException {
+    return dest.setFromRowNoNulls(row, joinKeys[alias], joinKeysObjectInspectors[alias], matchTracker);
+  }
+
   protected MapJoinKey getRefKey(byte alias) {
     // We assume that since we are joining on the same key, all tables would have either
     // optimized or non-optimized key; hence, we can pass any key in any table as reference.
@@ -341,6 +491,10 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
         for (byte pos = 0; pos < order.length; pos++) {
           if (pos != alias) {
             hashMapRowGetters[pos] = mapJoinTables[pos].createGetter(refKey);
+            if (isFullOuterMapJoin) {
+              matchTracker =
+                  ((ReusableGetAdaptorDirectAccess) hashMapRowGetters[pos]).createMatchTracker();
+            }
           }
         }
       }
@@ -367,13 +521,23 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
           ReusableGetAdaptor adaptor;
           if (firstSetKey == null) {
             adaptor = firstSetKey = hashMapRowGetters[pos];
-            joinResult = setMapJoinKey(firstSetKey, row, alias);
+            if (!isFullOuterMapJoin) {
+              // Normal case.
+              joinResult = setMapJoinKey(firstSetKey, row, alias);
+            } else {
+              // FULL OUTER MapJoin. We do not want keys with any NULLs to get tracked.
+              joinResult = setMapJoinKeyNoNulls(firstSetKey, row, alias, matchTracker);
+            }
           } else {
             // Keys for all tables are the same, so only the first has to deserialize them.
             adaptor = hashMapRowGetters[pos];
             joinResult = adaptor.setFromOther(firstSetKey);
           }
           MapJoinRowContainer rowContainer = adaptor.getCurrentRows();
+          if (joinResult != JoinUtil.JoinResult.MATCH) {
+            assert (rowContainer == null || !rowContainer.hasRows()) :
+                "Expecting an empty result set for no match";
+          }
           if (rowContainer != null && unwrapContainer[pos] != null) {
             Object[] currentKey = firstSetKey.getCurrentKey();
             rowContainer = unwrapContainer[pos].setInternal(rowContainer, currentKey);
@@ -383,13 +547,19 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
             if (!noOuterJoin) {
               // For Hybrid Grace Hash Join, during the 1st round processing,
               // we only keep the LEFT side if the row is not spilled
-              if (!conf.isHybridHashJoin() || hybridMapJoinLeftover
-                  || (!hybridMapJoinLeftover && joinResult != JoinUtil.JoinResult.SPILL)) {
+              if (!conf.isHybridHashJoin() || hybridMapJoinLeftover ||
+                  (joinResult != JoinUtil.JoinResult.SPILL && !bigTableRowSpilled)) {
                 joinNeeded = true;
                 storage[pos] = dummyObjVectors[pos];
+              } else {
+                joinNeeded = false;
               }
             } else {
               storage[pos] = emptyList;
+              if (pos != 0 && condn[pos - 1].getType() == JoinDesc.ANTI_JOIN) {
+                // For AntiJoin, we should call checkAndGenObject() when right side is empty.
+                joinNeeded = true;
+              }
             }
           } else {
             joinNeeded = true;
@@ -409,6 +579,7 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
       }
       if (joinNeeded) {
         List<Object> value = getFilteredValue(alias, row);
+        addToAliasFilterTags(alias, value, false);
         // Add the value to the ArrayList
         storage[alias].addRow(value);
         // generate the output records
@@ -422,7 +593,8 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
         }
       }
     } catch (Exception e) {
-      String msg = "Unexpected exception: " + e.getMessage();
+      String msg = "Unexpected exception from "
+          + this.getClass().getSimpleName() + " : " + e.getMessage();
       LOG.error(msg, e);
       throw new HiveException(msg, e);
     }
@@ -441,8 +613,134 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     bigTable.add(row);
   }
 
+  /*
+   * For FULL OUTER MapJoin, create a key match tracker on the Small Table.
+   */
+  private void createMatchTracker(MapJoinTableContainer smallTable) {
+    ReusableGetAdaptor hashMapRowGetter = smallTable.createGetter(null);
+    matchTracker =
+        ((ReusableGetAdaptorDirectAccess) hashMapRowGetter).createMatchTracker();
+    Preconditions.checkState(matchTracker != null);
+  }
+
+  private byte findSmallTable() {
+    byte smallTablePos = -1;
+    for (byte pos = 0; pos < mapJoinTables.length; pos++) {
+      if (pos != conf.getPosBigTable()) {
+        smallTablePos = pos;
+        break;
+      }
+    }
+    Preconditions.checkState(smallTablePos != -1);
+    return smallTablePos;
+  }
+
+  /*
+   * For FULL OUTER MapJoin, find the non matched Small Table keys and values and odd them to the
+   * join output result.
+   */
+  protected void generateFullOuterSmallTableNoMatches(byte smallTablePos,
+      MapJoinTableContainer substituteSmallTable) throws HiveException {
+
+    // FUTURE: Currently, in the MapJoinOperaotr, we only support FULL OUTER MapJoin for
+    // FUTURE  MapJoinBytesTableContainer.  NOTE: Vectorization code will override this method.
+
+    if (matchTracker == null) {
+
+      // When the process method isn't called (i.e. no rows), then we need to create the
+      // MatchTracker here.
+      //
+      // It will indicate no matches, of course.
+      //
+      createMatchTracker(substituteSmallTable);
+    }
+
+    boolean isSmallTableValuesOnly = false;
+    int[] smallTableValuesIndex = conf.getValueIndex(smallTablePos);
+    if (smallTableValuesIndex == null) {
+      List<Integer> valuesList = conf.getRetainList().get(smallTablePos);
+      smallTableValuesIndex =
+          ArrayUtils.toPrimitive(valuesList.toArray(new Integer[0]));
+      isSmallTableValuesOnly = true;
+    }
+    final int smallTableValuesIndexSize = smallTableValuesIndex.length;
+
+    // Our first output column for Small Table results is based on order.  (The Big Table columns
+    // will all be NULL).
+    final int firstOutputColumnNum = (posBigTable == (byte) 0 ? fullOuterBigTableRetainSize : 0);
+
+    /*
+     * Create iterator that produces each  non-matched Small Table key and a ReusableRowContainer
+     * the Small Table values.
+     */
+    NonMatchedSmallTableIterator nonMatchedIterator =
+        substituteSmallTable.createNonMatchedSmallTableIterator(matchTracker);
+    while (nonMatchedIterator.isNext()) {
+      List<Object> keyObjList = nonMatchedIterator.getCurrentKey();
+
+      MapJoinRowContainer values = nonMatchedIterator.getCurrentRows();
+      AbstractRowContainer.RowIterator<List<Object>> iter = values.rowIter();
+      for (List<Object> valueObjList = iter.first();
+          valueObjList != null;
+          valueObjList = iter.next()) {
+
+        // Form non-matched Small Table join result. We only fill in the Small Table columns,
+        // so the Big Table retained columns are NULLs from the new allocation.
+
+        Object[] row = new Object[fullOuterBigTableRetainSize + smallTableValuesIndexSize];
+        int outputColumnNum = firstOutputColumnNum;
+
+        if (isSmallTableValuesOnly) {
+          for (int i = 0; i < smallTableValuesIndexSize; i++) {
+            row[outputColumnNum++] = valueObjList.get(smallTableValuesIndex[i]);
+          }
+        } else {
+          for (int i = 0; i < smallTableValuesIndexSize; i++) {
+            final int index = smallTableValuesIndex[i];
+
+            if (index >= 0) {
+
+              // Zero and above numbers indicate a big table key is needed for
+              // small table result "area".
+
+              row[outputColumnNum++] = keyObjList.get(index);
+            } else {
+
+              // Negative numbers indicate a column to be (deserialize) read from the small table's
+              // LazyBinary value row.
+
+              int smallTableValueIndex = -index - 1;
+
+              row[outputColumnNum++] = valueObjList.get(smallTableValueIndex);
+            }
+          }
+        }
+
+        Object standardCopyRow =
+            ObjectInspectorUtils.copyToStandardObject(
+                row, outputObjInspector, ObjectInspectorCopyOption.WRITABLE);
+
+        // FUTURE: Support residual filters for non-equi joins.
+        internalForward(standardCopyRow, outputObjInspector);
+      }
+    }
+  }
+
   @Override
   public void closeOp(boolean abort) throws HiveException {
+
+    if (isFullOuterMapJoin) {
+
+      // FULL OUTER MapJoin: After matching the Big Table row keys against the Small Table, we now
+      // add any non matched Small Table key and values to the join output result.
+
+      // FUTURE: Currently, we only support FULL OUTER MapJoin for single condition MapJoins.
+      byte smallTablePos = findSmallTable();
+      generateFullOuterSmallTableNoMatches(
+          smallTablePos,
+          (MapJoinTableContainer) mapJoinTables[smallTablePos]);
+    }
+
     boolean spilled = false;
     for (MapJoinTableContainer container : mapJoinTables) {
       if (container != null) {
@@ -516,9 +814,7 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
         }
       }
 
-      if (isLogInfoEnabled) {
-        LOG.info("spilled: " + spilled + " abort: " + abort + ". Clearing spilled partitions.");
-      }
+      LOG.info("spilled: " + spilled + " abort: " + abort + ". Clearing spilled partitions.");
 
       // spilled tables are loaded always (no sharing), so clear it
       clearAllTableContainers();
@@ -528,9 +824,7 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     // in mapreduce case, we need to always clear up as mapreduce doesn't have object registry.
     if ((this.getExecContext() != null) && (this.getExecContext().getLocalWork() != null)
         && (this.getExecContext().getLocalWork().getInputFileChangeSensitive())) {
-      if (isLogInfoEnabled) {
-        LOG.info("MR: Clearing all map join table containers.");
-      }
+      LOG.info("MR: Clearing all map join table containers.");
       clearAllTableContainers();
     }
 
@@ -559,6 +853,7 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
       throws HiveException, IOException, SerDeException, ClassNotFoundException {
     for (byte pos = 0; pos < mapJoinTables.length; pos++) {
       if (pos != conf.getPosBigTable()) {
+        LOG.info("Going to reload hash partition " + partitionId);
         reloadHashTable(pos, partitionId);
       }
     }
@@ -583,6 +878,7 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
 
     // Merge the sidefile into the newly created hash table
     // This is where the spilling may happen again
+    LOG.info("Going to restore sidefile...");
     KeyValueContainer kvContainer = partition.getSidefileKVContainer();
     int rowCount = kvContainer.size();
     LOG.info("Hybrid Grace Hash Join: Number of rows restored from KeyValueContainer: " +
@@ -590,6 +886,12 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
 
     // Deserialize the on-disk hash table
     // We're sure this part is smaller than memory limit
+    if (rowCount <= 0) {
+      rowCount = 1024 * 1024; // Since rowCount is used later to instantiate a BytesBytesMultiHashMap
+                              // as the initialCapacity which cannot be 0, we provide a reasonable
+                              // positive number here
+    }
+    LOG.info("Going to restore hashmap...");
     BytesBytesMultiHashMap restoredHashMap = partition.getHashMapFromDisk(rowCount);
     rowCount += restoredHashMap.getNumValues();
     LOG.info("Hybrid Grace Hash Join: Deserializing spilled hash partition...");
@@ -605,20 +907,22 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
 
     KeyValueHelper writeHelper = container.getWriteHelper();
     while (kvContainer.hasNext()) {
-      ObjectPair<HiveKey, BytesWritable> pair = kvContainer.next();
-      Writable key = pair.getFirst();
-      Writable val = pair.getSecond();
+      Pair<HiveKey, BytesWritable> pair = kvContainer.next();
+      Writable key = pair.getLeft();
+      Writable val = pair.getRight();
       writeHelper.setKeyValue(key, val);
       restoredHashMap.put(writeHelper, -1);
     }
 
     container.setTotalInMemRowCount(container.getTotalInMemRowCount()
-        + restoredHashMap.getNumValues() + kvContainer.size());
+        + restoredHashMap.getNumValues());
     kvContainer.clear();
 
     spilledMapJoinTables[pos] = new MapJoinBytesTableContainer(restoredHashMap);
     spilledMapJoinTables[pos].setInternalValueOi(container.getInternalValueOi());
     spilledMapJoinTables[pos].setSortableSortOrders(container.getSortableSortOrders());
+    spilledMapJoinTables[pos].setNullMarkers(container.getNullMarkers());
+    spilledMapJoinTables[pos].setNotNullMarkers(container.getNotNullMarkers());
   }
 
   /**
@@ -633,6 +937,8 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
     // firstSmallTable has reference to the spilled big table rows.
     HashPartition partition = firstSmallTable.getHashPartitions()[partitionId];
     ObjectContainer bigTable = partition.getMatchfileObjContainer();
+    LOG.info("Hybrid Grace Hash Join: Going to process spilled big table rows in partition " +
+        partitionId + ". Number of rows: " + bigTable.size());
     while (bigTable.hasNext()) {
       Object row = bigTable.next();
       process(row, conf.getPosBigTable());
@@ -671,4 +977,47 @@ public class MapJoinOperator extends AbstractMapJoinOperator<MapJoinDesc> implem
   public OperatorType getType() {
     return OperatorType.MAPJOIN;
   }
+
+  protected boolean isInputFileChangeSensitive(ExecMapperContext mapContext) {
+    return !(mapContext == null
+        || mapContext.getLocalWork() == null
+        || mapContext.getLocalWork().getInputFileChangeSensitive() == false);
+  }
+
+  protected boolean canSkipReload(ExecMapperContext mapContext) {
+    return (this.hashTblInitedOnce && !isInputFileChangeSensitive(mapContext));
+  }
+
+  // If the loaded hash table is empty, for some conditions we can skip processing the big table rows.
+  protected boolean canSkipJoinProcessing(ExecMapperContext mapContext) {
+    if (!canSkipReload(mapContext)) {
+      return false;
+    }
+
+    JoinCondDesc[] joinConds = getConf().getConds();
+    if (joinConds.length > 0) {
+      for (JoinCondDesc joinCond : joinConds) {
+        if (joinCond.getType() != JoinDesc.INNER_JOIN) {
+          return false;
+        }
+      }
+    } else {
+      return false;
+    }
+
+    boolean skipJoinProcessing = false;
+    for (int idx = 0; idx < mapJoinTables.length; ++idx) {
+      if (idx == getConf().getPosBigTable()) {
+        continue;
+      }
+      MapJoinTableContainer mapJoinTable = mapJoinTables[idx];
+      if (mapJoinTable.size() == 0) {
+        // If any table is empty, an inner join involving the tables should yield 0 rows.
+        LOG.info("Hash table number " + idx + " is empty");
+        skipJoinProcessing = true;
+        break;
+      }
+    }
+    return skipJoinProcessing;
+  }
 }
