@@ -19,54 +19,20 @@
 package org.apache.hadoop.hive.ql;
 
 import java.io.File;
-import java.lang.reflect.Field;
-import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Comparator;
-import java.util.HashMap;
+import java.util.Collections;
 import java.util.List;
-import java.util.Map;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 
-import org.apache.curator.shaded.com.google.common.collect.Lists;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
-import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.common.FileUtils;
-import org.apache.hadoop.hive.common.ValidTxnList;
 import org.apache.hadoop.hive.conf.HiveConf;
-import org.apache.hadoop.hive.metastore.api.CommitTxnRequest;
-import org.apache.hadoop.hive.metastore.api.CompactionRequest;
-import org.apache.hadoop.hive.metastore.api.CompactionType;
-import org.apache.hadoop.hive.metastore.api.MetaException;
-import org.apache.hadoop.hive.metastore.api.OpenTxnRequest;
-import org.apache.hadoop.hive.metastore.api.OpenTxnsResponse;
-import org.apache.hadoop.hive.metastore.api.ShowCompactRequest;
-import org.apache.hadoop.hive.metastore.api.ShowCompactResponse;
-import org.apache.hadoop.hive.metastore.api.ShowCompactResponseElement;
-import org.apache.hadoop.hive.metastore.txn.AcidCompactionHistoryService;
-import org.apache.hadoop.hive.metastore.txn.TxnDbUtil;
-import org.apache.hadoop.hive.metastore.txn.TxnStore;
-import org.apache.hadoop.hive.metastore.txn.TxnUtils;
-import org.apache.hadoop.hive.ql.io.AcidOutputFormat;
-import org.apache.hadoop.hive.ql.io.BucketCodec;
-import org.apache.hadoop.hive.ql.io.HiveInputFormat;
-import org.apache.hadoop.hive.ql.processors.CommandProcessorResponse;
-import org.apache.hadoop.hive.ql.session.SessionState;
-import org.apache.hadoop.hive.metastore.txn.AcidOpenTxnsCounterService;
-import org.apache.hadoop.hive.ql.txn.compactor.Cleaner;
-import org.apache.hadoop.hive.ql.txn.compactor.Initiator;
-import org.apache.hadoop.hive.ql.txn.compactor.Worker;
-import org.junit.After;
+import org.apache.hadoop.hive.metastore.utils.TestTxnDbUtil;
+import org.apache.hadoop.hive.ql.io.AcidUtils;
 import org.junit.Assert;
 import org.junit.Before;
-import org.junit.Ignore;
-import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.ExpectedException;
-import org.junit.rules.TestName;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -80,9 +46,9 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     File.separator + TestTxnCommands.class.getCanonicalName()
     + "-" + System.currentTimeMillis()
   ).getPath().replaceAll("\\\\", "/");
-  protected static final String TEST_WAREHOUSE_DIR = TEST_DATA_DIR + "/warehouse";
+
   @Override
-  String getTestDataDir() {
+  protected String getTestDataDir() {
     return TEST_DATA_DIR;
   }
 
@@ -98,9 +64,6 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     public String toString() {
       return name;
     }
-    String getPartitionColumns() {
-      return partitionColumns;
-    }
     TableExtended(String name) {
       this(name, null);
     }
@@ -109,7 +72,13 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
       this.partitionColumns = partitionColumns;
     }
   }
-  
+
+  @Override
+  void initHiveConf() {
+    super.initHiveConf();
+    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVE_ACID_TRUNCATE_USE_BASE, false);
+  }
+
   @Override
   @Before
   public void setUp() throws Exception {
@@ -118,10 +87,9 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
   }
   
   void setUpInternalExtended(boolean isOrcFormat) throws Exception {
-    hiveConf.setBoolVar(HiveConf.ConfVars.DYNAMICPARTITIONING, true);
-    hiveConf.setVar(HiveConf.ConfVars.DYNAMICPARTITIONINGMODE, "nonstrict");
-    hiveConf.setVar(HiveConf.ConfVars.HIVEMAPREDMODE, "nonstrict");
-    hiveConf.setVar(HiveConf.ConfVars.HIVEFETCHTASKCONVERSION, "none");
+    hiveConf.setBoolVar(HiveConf.ConfVars.DYNAMIC_PARTITIONING, true);
+    hiveConf.setVar(HiveConf.ConfVars.HIVE_MAPRED_MODE, "nonstrict");
+    hiveConf.setVar(HiveConf.ConfVars.HIVE_FETCH_TASK_CONVERSION, "none");
     hiveConf.set(HiveConf.ConfVars.HIVE_SUPPORT_CONCURRENCY.varname, "true");
     hiveConf.setVar(HiveConf.ConfVars.HIVE_TXN_MANAGER, "org.apache.hadoop.hive.ql.lockmgr.DbTxnManager");
 
@@ -136,48 +104,14 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
       runStatementOnDriver("create table " + TableExtended.MMTBLPART + "(a int, b int) partitioned by (p string) stored as orc TBLPROPERTIES ('transactional'='true', 'transactional_properties'='insert_only')");
     }
   }
+
+  @Override
   protected void dropTables() throws Exception {
     super.dropTables();
-    for(TestTxnCommandsForMmTable.TableExtended t : TestTxnCommandsForMmTable.TableExtended.values()) {
+    for (TestTxnCommandsForMmTable.TableExtended t : TestTxnCommandsForMmTable.TableExtended.values()) {
       runStatementOnDriver("drop table if exists " + t);
     }
   }
-  /**
-   * Test compaction for Micro-managed table
-   * 1. Regular compaction shouldn't impact any valid subdirectories of MM tables
-   * 2. Compactions will only remove subdirectories for aborted transactions of MM tables, if any
-   * @throws Exception
-   */
-  @Test
-  public void testMmTableCompaction() throws Exception {
-    // 1. Insert some rows into MM table
-    runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(1,2)");
-    runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(3,4)");
-    // There should be 2 delta directories
-    verifyDirAndResult(2);
-
-    // 2. Perform a MINOR compaction. Since nothing was aborted, subdirs should stay.
-    runStatementOnDriver("alter table "+ TableExtended.MMTBL + " compact 'MINOR'");
-    runWorker(hiveConf);
-    verifyDirAndResult(2);
-
-    // 3. Let a transaction be aborted
-    hiveConf.setBoolVar(HiveConf.ConfVars.HIVETESTMODEROLLBACKTXN, true);
-    runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(5,6)");
-    hiveConf.setBoolVar(HiveConf.ConfVars.HIVETESTMODEROLLBACKTXN, false);
-    // There should be 3 delta directories. The new one is the aborted one.
-    verifyDirAndResult(3);
-
-    // 4. Perform a MINOR compaction again. This time it will remove the subdir for aborted transaction.
-    runStatementOnDriver("alter table "+ TableExtended.MMTBL + " compact 'MINOR'");
-    runWorker(hiveConf);
-    // The worker should remove the subdir for aborted transaction
-    verifyDirAndResult(2);
-
-    // 5. Run Cleaner. Shouldn't impact anything.
-    runCleaner(hiveConf);
-    verifyDirAndResult(2);
-  }
 
   /**
    * Test a scenario, on a micro-managed table, where an IOW comes in
@@ -193,7 +127,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     // 1. Insert two rows to an MM table
     runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(1,2)");
     runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(3,4)");
-    status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+    status = fs.listStatus(new Path(getWarehouseDir() + "/" +
         (TableExtended.MMTBL).toString().toLowerCase()), FileUtils.STAGING_DIR_PATH_FILTER);
     // There should be 2 delta dirs in the location
     Assert.assertEquals(2, status.length);
@@ -206,7 +140,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     runStatementOnDriver("insert into " + Table.NONACIDORCTBL + "(a,b) values(5,6),(7,8)");
     // Insert overwrite MM table from source table
     runStatementOnDriver("insert overwrite table " + TableExtended.MMTBL + " select a,b from " + Table.NONACIDORCTBL);
-    status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+    status = fs.listStatus(new Path(getWarehouseDir() + "/" +
         (TableExtended.MMTBL).toString().toLowerCase()), FileUtils.STAGING_DIR_PATH_FILTER);
     // There should be 2 delta dirs, plus 1 base dir in the location
     Assert.assertEquals(3, status.length);
@@ -250,7 +184,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     String[] pStrings = {"/p=odd", "/p=even"};
 
     for(int i=0; i < pStrings.length; i++) {
-      status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+      status = fs.listStatus(new Path(getWarehouseDir() + "/" +
           (TableExtended.MMTBLPART).toString().toLowerCase() + pStrings[i]), FileUtils.STAGING_DIR_PATH_FILTER);
       // There should be 1 delta dir per partition location
       Assert.assertEquals(1, status.length);
@@ -282,7 +216,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     String[] baseDirs = {"", ""};
     int deltaCount = 0;
     for(int h=0; h < pStrings.length; h++) {
-      status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+      status = fs.listStatus(new Path(getWarehouseDir() + "/" +
           (TableExtended.MMTBLPART).toString().toLowerCase() + pStrings[h]), FileUtils.STAGING_DIR_PATH_FILTER);
       // There should be 1 delta dir, plus a base dir in the location
       Assert.assertEquals(2, status.length);
@@ -293,7 +227,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
         } else {
           sawBase = true;
           baseDirs[h] = dirName;
-          Assert.assertTrue(baseDirs[i].matches("base_.*"));
+          Assert.assertTrue(baseDirs[h].matches("base_.*"));
         }
       }
       Assert.assertEquals(1, deltaCount);
@@ -320,7 +254,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     runWorker(hiveConf);
 
     for(int h=0; h < pStrings.length; h++) {
-      status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+      status = fs.listStatus(new Path(getWarehouseDir() + "/" +
           (TableExtended.MMTBLPART).toString().toLowerCase() + pStrings[h]), FileUtils.STAGING_DIR_PATH_FILTER);
       // There should be 2 delta dirs, plus a base dir in the location
       Assert.assertEquals(2, status.length);
@@ -353,7 +287,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     // There should be only 1 directory left: base_xxxxxxx.
     // The delta dirs should have been cleaned up.
     for(int h=0; h < pStrings.length; h++) {
-      status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+      status = fs.listStatus(new Path(getWarehouseDir() + "/" +
           (TableExtended.MMTBLPART).toString().toLowerCase() + pStrings[h]), FileUtils.STAGING_DIR_PATH_FILTER);
       Assert.assertEquals(1, status.length);
       Assert.assertTrue(status[0].getPath().getName().matches("base_.*"));
@@ -385,7 +319,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     String[] pStrings = {"/p=odd", "/p=even"};
 
     for(int i=0; i < pStrings.length; i++) {
-      status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+      status = fs.listStatus(new Path(getWarehouseDir() + "/" +
           (TableExtended.MMTBLPART).toString().toLowerCase() + pStrings[i]), FileUtils.STAGING_DIR_PATH_FILTER);
       // There should be 1 delta dir per partition location
       Assert.assertEquals(1, status.length);
@@ -407,7 +341,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     String[] baseDirs = {"", ""};
     int deltaCount = 0;
     for(int h=0; h < pStrings.length; h++) {
-      status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+      status = fs.listStatus(new Path(getWarehouseDir() + "/" +
           (TableExtended.MMTBLPART).toString().toLowerCase() + pStrings[h]), FileUtils.STAGING_DIR_PATH_FILTER);
       // There should be 1 delta dir, plus a base dir in the location
       Assert.assertEquals(2, status.length);   // steve
@@ -452,7 +386,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     // 1. Insert two rows to an MM table
     runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(1,2)");
     runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(3,4)");
-    status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+    status = fs.listStatus(new Path(getWarehouseDir() + "/" +
         (TableExtended.MMTBL).toString().toLowerCase()), FileUtils.STAGING_DIR_PATH_FILTER);
     // There should be 2 delta dirs in the location
     Assert.assertEquals(2, status.length);
@@ -462,7 +396,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
 
     // 2. Insert Overwrite.
     int[][] values = {{1,2},{2,4},{5,6},{6,8},{9,10}};
-    runStatementOnDriver("insert into " + Table.NONACIDORCTBL + TestTxnCommands2.makeValuesClause(values));
+    runStatementOnDriver("insert into " + Table.NONACIDORCTBL + makeValuesClause(values));
 
     runStatementOnDriver("insert overwrite table " + TableExtended.MMTBL + " select a,b from " + Table.NONACIDORCTBL + " where a between 1 and 3 union all select a,b from " + Table.NONACIDORCTBL + " where a between 5 and 7");
 
@@ -471,7 +405,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     String baseDir = "";
     int deltaCount = 0;
 
-    status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+    status = fs.listStatus(new Path(getWarehouseDir() + "/" +
       (TableExtended.MMTBL).toString().toLowerCase()), FileUtils.STAGING_DIR_PATH_FILTER);
     // There should be 2 delta dirs, plus a base dir in the location
     Assert.assertEquals(3, status.length);
@@ -504,7 +438,7 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     Assert.assertEquals(stringifyValues(rExpected), rs);
 
     // Verify resulting dirs.
-    status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
+    status = fs.listStatus(new Path(getWarehouseDir() + "/" +
       (TableExtended.MMTBL).toString().toLowerCase()), FileUtils.STAGING_DIR_PATH_FILTER);
     // There should be one base dir in the location
     Assert.assertEquals(1, status.length);
@@ -537,132 +471,192 @@ public class TestTxnCommandsForMmTable extends TxnCommandsBaseForTests {
     // There should be 2 delta directories
     verifyDirAndResult(2);
 
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from COMPLETED_TXN_COMPONENTS"),
-            2, TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from COMPLETED_TXN_COMPONENTS"));
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXNS"),
-            0, TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXNS"));
+    Assert.assertEquals(TestTxnDbUtil.queryToString(hiveConf, "select * from COMPLETED_TXN_COMPONENTS"),
+            2, TestTxnDbUtil.countQueryAgent(hiveConf, "select count(*) from COMPLETED_TXN_COMPONENTS"));
 
-    // Initiate a minor compaction request on the table.
+    // Initiate a major compaction request on the table.
     runStatementOnDriver("alter table " + TableExtended.MMTBL  + " compact 'MAJOR'");
 
     // Run worker.
     runWorker(hiveConf);
+    verifyDirAndResult(2, true);
 
     // Run Cleaner.
     runCleaner(hiveConf);
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from COMPLETED_TXN_COMPONENTS"),
+    Assert.assertEquals(TestTxnDbUtil.queryToString(hiveConf, "select * from COMPLETED_TXN_COMPONENTS"),
             0,
-            TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from COMPLETED_TXN_COMPONENTS"));
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXNS"),
-            0, TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXNS"));
+            TestTxnDbUtil.countQueryAgent(hiveConf, "select count(*) from COMPLETED_TXN_COMPONENTS"));
+    verifyDirAndResult(0, true);
   }
 
   @Test
-  public void testSnapshotIsolationWithAbortedTxnOnMmTable() throws Exception {
+  public void testImpalaTruncatedMmTableVectorized() throws Exception {
+    testImpalaTruncatedMmTable(true);
+  }
 
-    // Insert two rows into the table.
+  @Test
+  public void testImpalaTruncatedMmTableNonVectorized() throws Exception {
+    testImpalaTruncatedMmTable(false);
+  }
+
+  /**
+   * Impala truncates insert-only tables by writing a base directory (like insert overwrite) containing a completely
+   * empty file. Make sure that Hive reads these bases correctly.
+   *
+   * @throws Exception
+   */
+  private void testImpalaTruncatedMmTable(boolean vectorized) throws Exception {
+    if (!vectorized) {
+      d.getConf().setBoolVar(HiveConf.ConfVars.HIVE_VECTORIZATION_ENABLED, false);
+    }
+    FileSystem fs = FileSystem.get(hiveConf);
+    FileStatus[] status;
+    Path tblLocation = new Path(getWarehouseDir() + "/" +
+        (TableExtended.MMTBL).toString().toLowerCase());
+
+    // 1. Insert two rows to an MM table
+    runStatementOnDriver("drop table " + TableExtended.MMTBL);
+    runStatementOnDriver("create table " + TableExtended.MMTBL + "(a int,b int) stored as parquet "
+        + "TBLPROPERTIES ('transactional'='true', 'transactional_properties'='insert_only')");
     runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(1,2)");
     runStatementOnDriver("insert into " + TableExtended.MMTBL + "(a,b) values(3,4)");
-    // There should be 2 delta directories
-    verifyDirAndResult(2);
+    status = fs.listStatus(tblLocation, FileUtils.STAGING_DIR_PATH_FILTER);
+    // There should be 2 delta dirs in the location
+    Assert.assertEquals(2, status.length);
+    for (int i = 0; i < status.length; i++) {
+      Assert.assertTrue(status[i].getPath().getName().matches("delta_.*"));
+    }
 
-    // Initiate a minor compaction request on the table.
-    runStatementOnDriver("alter table " + TableExtended.MMTBL  + " compact 'MINOR'");
+    // 2. Simulate Impala truncating the table: write a base dir (base_0000003) containing a file with no data. We
+    // have to delete this file (it's not completely empty, it contains metadata) and create completely empty file
+    runStatementOnDriver("insert overwrite  table " + TableExtended.MMTBL + " select * from "
+        + TableExtended.MMTBL + " where 1=2");
+    status = fs.listStatus(tblLocation, FileUtils.STAGING_DIR_PATH_FILTER);
+    // There should be 2 delta dirs, plus 1 base dir in the location
+    Assert.assertEquals(3, status.length);
+    verifyDir(2, true);
+    Path basePath = new Path(tblLocation, "base_0000003");
+    Assert.assertTrue("Deleting file under base failed", fs.delete(new Path(basePath, "000000_0")));
+    fs.create(new Path(basePath, "empty"));
 
-    // Run Compaction Worker to do compaction.
-    // But we do not compact a MM table but only transit the compaction request to
-    // "ready for cleaning" state in this case.
-    runWorker(hiveConf);
-    verifyDirAndResult(2);
-
-    // Start an INSERT statement transaction and roll back this transaction.
-    hiveConf.setBoolVar(HiveConf.ConfVars.HIVETESTMODEROLLBACKTXN, true);
-    runStatementOnDriver("insert into " + TableExtended.MMTBL  + " values (5, 6)");
-    hiveConf.setBoolVar(HiveConf.ConfVars.HIVETESTMODEROLLBACKTXN, false);
-    // There should be 3 delta directories. The new one is the aborted one.
-    verifyDirAndResult(3);
-
-    // Execute SELECT statement and verify the result set (should be two rows).
-    int[][] expected = new int[][] {{1, 2}, {3, 4}};
+    // 3. Verify query result. Selecting from a truncated table should return nothing.
     List<String> rs = runStatementOnDriver("select a,b from " + TableExtended.MMTBL + " order by a,b");
-    Assert.assertEquals(stringifyValues(expected), rs);
+    Assert.assertEquals(Collections.emptyList(), rs);
 
-    // Run Cleaner.
-    // This run doesn't do anything for the above aborted transaction since
-    // the current compaction request entry in the compaction queue is updated
-    // to have highest_write_id when the worker is run before the aborted
-    // transaction. Specifically the id is 2 for the entry but the aborted
-    // transaction has 3 as writeId. This run does transition the entry
-    // "successful".
-    runCleaner(hiveConf);
-    verifyDirAndResult(3);
-
-    // Execute SELECT and verify that aborted operation is not counted for MM table.
-    rs = runStatementOnDriver("select a,b from " + TableExtended.MMTBL + " order by a,b");
-    Assert.assertEquals(stringifyValues(expected), rs);
-
-    // Run initiator to execute CompactionTxnHandler.cleanEmptyAbortedTxns()
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXNS"),
-            1, TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXNS"));
-    Initiator i = new Initiator();
-    i.setThreadId((int)i.getId());
-    i.setConf(hiveConf);
-    AtomicBoolean stop = new AtomicBoolean(true);
-    i.init(stop, new AtomicBoolean());
-    i.run();
-    // This run of Initiator doesn't add any compaction_queue entry
-    // since we only have one MM table with data - we don't compact MM tables.
-    verifyDirAndResult(3);
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXNS"),
-            1, TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXNS"));
-
-    // Execute SELECT statement and verify that aborted INSERT statement is not counted.
-    rs = runStatementOnDriver("select a,b from " + TableExtended.MMTBL + " order by a,b");
-    Assert.assertEquals(stringifyValues(expected), rs);
-
-    // Initiate a minor compaction request on the table.
-    runStatementOnDriver("alter table " + TableExtended.MMTBL  + " compact 'MINOR'");
-
-    // Run worker to delete aborted transaction's delta directory.
+    // 4. Perform a major compaction. Cleaner should remove the 2 delta dirs.
+    runStatementOnDriver("alter table "+ TableExtended.MMTBL + " compact 'MAJOR'");
     runWorker(hiveConf);
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXNS"),
-            1, TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXNS"));
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXN_COMPONENTS"),
-            1,
-            TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXN_COMPONENTS"));
-    verifyDirAndResult(2);
-
-    // Run Cleaner to delete rows for the aborted transaction
-    // from TXN_COMPONENTS.
     runCleaner(hiveConf);
-
-    // Run initiator to clean the row fro the aborted transaction from TXNS.
-    i.run();
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXNS"),
-            0, TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXNS"));
-    Assert.assertEquals(TxnDbUtil.queryToString(hiveConf, "select * from TXN_COMPONENTS"),
-            0,
-            TxnDbUtil.countQueryAgent(hiveConf, "select count(*) from TXN_COMPONENTS"));
+    verifyDir(0, true);
+    rs = runStatementOnDriver("select a,b from " + TableExtended.MMTBL + " order by a,b");
+    Assert.assertEquals(Collections.emptyList(), rs);
+    if (!vectorized) {
+      d.getConf().setBoolVar(HiveConf.ConfVars.HIVE_VECTORIZATION_ENABLED, true);
+    }
   }
 
   private void verifyDirAndResult(int expectedDeltas) throws Exception {
-    FileSystem fs = FileSystem.get(hiveConf);
-    // Verify the content of subdirs
-    FileStatus[] status = fs.listStatus(new Path(TEST_WAREHOUSE_DIR + "/" +
-        (TableExtended.MMTBL).toString().toLowerCase()), FileUtils.HIDDEN_FILES_PATH_FILTER);
-    int sawDeltaTimes = 0;
-    for (int i = 0; i < status.length; i++) {
-      Assert.assertTrue(status[i].getPath().getName().matches("delta_.*"));
-      sawDeltaTimes++;
-      FileStatus[] files = fs.listStatus(status[i].getPath(), FileUtils.HIDDEN_FILES_PATH_FILTER);
-      Assert.assertEquals(1, files.length);
-      Assert.assertTrue(files[0].getPath().getName().equals("000000_0"));
-    }
-    Assert.assertEquals(expectedDeltas, sawDeltaTimes);
+    verifyDirAndResult(expectedDeltas, false);
+  }
+  private void verifyDirAndResult(int expectedDeltas, boolean expectBaseDir) throws Exception {
+    verifyDir(expectedDeltas, expectBaseDir);
 
     // Verify query result
     int [][] resultData = new int[][] {{1,2}, {3,4}};
     List<String> rs = runStatementOnDriver("select a,b from " + TableExtended.MMTBL + " order by a,b");
     Assert.assertEquals(stringifyValues(resultData), rs);
   }
+
+  private void verifyDir(int expectedDeltas, boolean expectBaseDir) throws Exception {
+    FileSystem fs = FileSystem.get(hiveConf);
+    // Verify the content of subdirs
+    FileStatus[] status = fs.listStatus(new Path(getWarehouseDir() + "/" +
+        (TableExtended.MMTBL).toString().toLowerCase()), FileUtils.HIDDEN_FILES_PATH_FILTER);
+    int sawDeltaTimes = 0;
+    int sawBaseTimes = 0;
+    for (int i = 0; i < status.length; i++) {
+      if (status[i].getPath().getName().matches("delta_.*")) {
+        sawDeltaTimes++;
+        FileStatus[] files = fs.listStatus(status[i].getPath(), FileUtils.HIDDEN_FILES_PATH_FILTER);
+        Assert.assertEquals(1, files.length);
+        Assert.assertEquals("000000_0", files[0].getPath().getName());
+      } else {
+        sawBaseTimes++;
+      }
+    }
+
+    Assert.assertEquals(expectedDeltas, sawDeltaTimes);
+    if (expectBaseDir) {
+      Assert.assertEquals("1 base directory expected", 1, sawBaseTimes);
+    } else {
+      Assert.assertEquals("0 base directories expected", 0, sawBaseTimes);
+    }
+  }
+
+  @Test
+  public void testTruncateWithBase() throws Exception {
+    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVE_ACID_TRUNCATE_USE_BASE, true);
+
+    runStatementOnDriver("insert into " + TableExtended.MMTBL + " values(1,2),(3,4)");
+    runStatementOnDriver("truncate table " + TableExtended.MMTBL);
+
+    FileSystem fs = FileSystem.get(hiveConf);
+    FileStatus[] stat =
+        fs.listStatus(new Path(getWarehouseDir(), TableExtended.MMTBL.toString().toLowerCase()), AcidUtils.baseFileFilter);
+    if (1 != stat.length) {
+      Assert.fail("Expecting 1 base and found " + stat.length + " files " + Arrays.toString(stat));
+    }
+    String name = stat[0].getPath().getName();
+    Assert.assertEquals("base_0000002", name);
+
+    List<String> r = runStatementOnDriver("select * from " + TableExtended.MMTBL);
+    Assert.assertEquals(0, r.size());
+  }
+
+  @Test
+  public void testTruncateWithBaseAllPartition() throws Exception {
+    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVE_ACID_TRUNCATE_USE_BASE, true);
+
+    runStatementOnDriver("insert into " + TableExtended.MMTBLPART + " partition(p='a') values(1,2),(3,4)");
+    runStatementOnDriver("insert into " + TableExtended.MMTBLPART + " partition(p='b') values(1,2),(3,4)");
+    runStatementOnDriver("truncate table " + TableExtended.MMTBLPART);
+
+    FileSystem fs = FileSystem.get(hiveConf);
+    FileStatus[] stat =
+        fs.listStatus(new Path(getWarehouseDir(), TableExtended.MMTBLPART.toString().toLowerCase() + "/p=a"), AcidUtils.baseFileFilter);
+    if (1 != stat.length) {
+      Assert.fail("Expecting 1 base and found " + stat.length + " files " + Arrays.toString(stat));
+    }
+    String name = stat[0].getPath().getName();
+    Assert.assertEquals("base_0000003", name);
+
+    List<String> r = runStatementOnDriver("select * from " + TableExtended.MMTBLPART);
+    Assert.assertEquals(0, r.size());
+  }
+
+  @Test
+  public void testTruncateWithBaseOnePartition() throws Exception {
+    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVE_ACID_TRUNCATE_USE_BASE, true);
+
+    runStatementOnDriver("insert into " + TableExtended.MMTBLPART + " partition(p='a') values(1,2),(3,4)");
+    runStatementOnDriver("insert into " + TableExtended.MMTBLPART+ " partition(p='b') values(5,5),(4,4)");
+    runStatementOnDriver("truncate table " + TableExtended.MMTBLPART + " partition(p='b')");
+
+    FileSystem fs = FileSystem.get(hiveConf);
+    FileStatus[] stat =
+        fs.listStatus(new Path(getWarehouseDir(), TableExtended.MMTBLPART.toString().toLowerCase() + "/p=b"), AcidUtils.baseFileFilter);
+    if (1 != stat.length) {
+      Assert.fail("Expecting 1 base and found " + stat.length + " files " + Arrays.toString(stat));
+    }
+    String name = stat[0].getPath().getName();
+    Assert.assertEquals("base_0000003", name);
+    stat =
+        fs.listStatus(new Path(getWarehouseDir(), TableExtended.MMTBLPART.toString().toLowerCase() + "/p=a"), AcidUtils.deltaFileFilter);
+    if (1 != stat.length) {
+      Assert.fail("Expecting 1 delta and found " + stat.length + " files " + Arrays.toString(stat));
+    }
+
+    List<String> r = runStatementOnDriver("select * from " + TableExtended.MMTBLPART);
+    Assert.assertEquals(2, r.size());
+  }
 }
