@@ -19,7 +19,6 @@ package org.apache.hadoop.hive.ql;
 
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.metastore.conf.MetastoreConf;
-import org.apache.hadoop.hive.metastore.tools.HiveMetaTool;
 import org.junit.Assert;
 import org.junit.Ignore;
 import org.junit.Test;
@@ -39,7 +38,7 @@ public class TestTxnExIm extends TxnCommandsBaseForTests {
   ).getPath().replaceAll("\\\\", "/");
 
   @Override
-  String getTestDataDir() {
+  protected String getTestDataDir() {
     return TEST_DATA_DIR;
   }
 
@@ -127,7 +126,6 @@ public class TestTxnExIm extends TxnCommandsBaseForTests {
    */
   @Test
   public void testExportPart() throws Exception {
-    hiveConf.setVar(HiveConf.ConfVars.DYNAMICPARTITIONINGMODE, "nonstrict");
     int[][] rows1 = {{1, 2, 1}, {3, 4, 2}};
     runStatementOnDriver("drop table if exists T");
     runStatementOnDriver("drop table if exists TImport ");
@@ -155,7 +153,6 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1519423568221/
    */
   @Test
   public void testExportPartPartial() throws Exception {
-    hiveConf.setVar(HiveConf.ConfVars.DYNAMICPARTITIONINGMODE, "nonstrict");
     int[][] rows1 = {{1, 2, 1, 1}, {3, 4, 2, 2}, {5, 6, 1, 2}, {7, 8, 2, 2}};
     runStatementOnDriver("drop table if exists T");
     runStatementOnDriver("drop table if exists TImport ");
@@ -215,7 +212,6 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1521148657811/
    */
   @Test
   public void testExportPartPartial2() throws Exception {
-    hiveConf.setVar(HiveConf.ConfVars.DYNAMICPARTITIONINGMODE, "nonstrict");
     int[][] rows1 = {{1, 2, 1, 1}, {3, 4, 2, 2}, {5, 6, 1, 2}, {7, 8, 2, 2}};
     runStatementOnDriver("drop table if exists T");
     runStatementOnDriver("drop table if exists TImport ");
@@ -233,7 +229,6 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1521148657811/
   }
   @Test
   public void testExportPartPartial3() throws Exception {
-    hiveConf.setVar(HiveConf.ConfVars.DYNAMICPARTITIONINGMODE, "nonstrict");
     int[][] rows1 = {{1, 1, 1, 2}, {3, 2, 3, 8}, {5, 1, 2, 6}, {7, 2, 2, 8}};
     runStatementOnDriver("drop table if exists T");
     runStatementOnDriver("drop table if exists TImport ");
@@ -343,19 +338,19 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1521148657811/
             "t/delta_0000001_0000001_0000/000000_0"},
         {"{\"writeid\":1,\"bucketid\":536870912,\"rowid\":1}\t3\t4",
             "t/delta_0000001_0000001_0000/000000_0"},
-        {"{\"writeid\":2,\"bucketid\":536870912,\"rowid\":0}\t0\t6",
-            "t/delta_0000002_0000002_0000/bucket_00000"}};
+        {"{\"writeid\":2,\"bucketid\":536870913,\"rowid\":0}\t0\t6",
+            "t/delta_0000002_0000002_0001/bucket_00000_0"}};
     checkResult(expected2, testQuery, isVectorized, "update imported table");
 
     runStatementOnDriver("alter table T compact 'minor'");
     TestTxnCommands2.runWorker(hiveConf);
     String[][] expected3 = new String[][] {
         {"{\"writeid\":1,\"bucketid\":536870912,\"rowid\":0}\t1\t2",
-            "t/delta_0000001_0000002/bucket_00000"},
+            ".*t/delta_0000001_0000002_v000002[6-7]/bucket_00000"},
         {"{\"writeid\":1,\"bucketid\":536870912,\"rowid\":1}\t3\t4",
-            "t/delta_0000001_0000002/bucket_00000"},
-        {"{\"writeid\":2,\"bucketid\":536870912,\"rowid\":0}\t0\t6",
-            "t/delta_0000001_0000002/bucket_00000"}};
+            ".*t/delta_0000001_0000002_v000002[6-7]/bucket_00000"},
+        {"{\"writeid\":2,\"bucketid\":536870913,\"rowid\":0}\t0\t6",
+            ".*t/delta_0000001_0000002_v000002[6-7]/bucket_00000"}};
     checkResult(expected3, testQuery, isVectorized, "minor compact imported table");
 
   }
@@ -384,7 +379,7 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1521148657811/
         "select ROW__ID, a, b, INPUT__FILE__NAME from T order by ROW__ID";
     String[][] expected = new String[][] {
         {"{\"writeid\":1,\"bucketid\":536870912,\"rowid\":0}\t0\t0",
-            "t/p=10/delta_0000001_0000001_0000/bucket_00000"},
+            "t/p=10/delta_0000001_0000001_0000/bucket_00000_0"},
         {"{\"writeid\":2,\"bucketid\":536870912,\"rowid\":0}\t3\t4",
             "t/p=11/delta_0000002_0000002_0000/000000_0"},
         {"{\"writeid\":3,\"bucketid\":536870912,\"rowid\":0}\t5\t6",
@@ -392,6 +387,58 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1521148657811/
     checkResult(expected, testQuery, isVectorized, "import existing table");
   }
 
+  @Test
+  public void testImportPartitionedOrc() throws Exception {
+    // Clear and drop table T,Tstage
+    runStatementOnDriver("drop table if exists T");
+    runStatementOnDriver("drop table if exists Tstage");
+
+    // Create source table - Tstage
+    runStatementOnDriver("create table Tstage (a int, b int) partitioned by (p int) stored"
+        + " as orc tblproperties('transactional'='true')");
+
+    // This creates an ORC data file with correct schema under table root
+    runStatementOnDriver("insert into Tstage values(1,2,10),(3,4,11),(5,6,12)");
+    final int[][] rows = { { 3 } };
+
+    // Check Partitions statistics
+    List<String> rsTstagePartitionsProperties = runStatementOnDriver("show partitions Tstage");
+    for (String rsTstagePartition : rsTstagePartitionsProperties) {
+      List<String> rsPartitionProperties =
+          runStatementOnDriver("describe formatted Tstage partition(" + rsTstagePartition + ")");
+      Assert.assertEquals("COLUMN_STATS_ACCURATE of partition " + rsTstagePartition + " of Tstage table", true,
+          rsPartitionProperties.contains("\tCOLUMN_STATS_ACCURATE\t{\\\"BASIC_STATS\\\":\\\"true\\\"}"));
+      Assert.assertEquals(" of partition " + rsTstagePartition + " of Tstage table", true,
+          rsPartitionProperties.contains("\tnumRows             \t1                   "));
+    }
+
+    // Now we have an archive Tstage with 3 partitions
+    runStatementOnDriver("export table Tstage to '" + getWarehouseDir() + "/1'");
+
+    // Load T
+    runStatementOnDriver("import table T from '" + getWarehouseDir() + "/1'");
+
+    // Check basic stats in tblproperties of T
+    List<String> rsTProperties = runStatementOnDriver("show tblproperties T");
+    Assert.assertEquals("COLUMN_STATS_ACCURATE of T table", false,
+        rsTProperties.contains("COLUMN_STATS_ACCURATE\t{\"BASIC_STATS\":\"true\"}"));
+    Assert.assertEquals("numRows of T table", false, rsTProperties.contains("numRows\t3"));
+
+    // Check Partitions statistics of T
+    List<String> rsTPartitionsProperties = runStatementOnDriver("show partitions T");
+    for (String rsTPartition : rsTPartitionsProperties) {
+      List<String> rsPartitionProperties = runStatementOnDriver("describe formatted T partition(" + rsTPartition + ")");
+      Assert.assertEquals("COLUMN_STATS_ACCURATE of partition " + rsTPartition + " of T table", false,
+          rsPartitionProperties.contains("\tCOLUMN_STATS_ACCURATE\t{\\\"BASIC_STATS\\\":\\\"true\\\"}"));
+      Assert.assertEquals(" of partition " + rsTPartition + " of T table", false,
+          rsPartitionProperties.contains("\tnumRows             \t1                   "));
+    }
+
+    // Verify the count(*) output
+    List<String> rs = runStatementOnDriver("select count(*) from T");
+    Assert.assertEquals("Rowcount of imported table", TestTxnCommands2.stringifyValues(rows), rs);
+  }
+
   /**
    * test selective partitioned import where target table needs to be created.
    * export is made from acid table so that target table is created as acid
@@ -537,9 +584,9 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1521148657811/
     runStatementOnDriver("create table T (a int, b int)");
     runStatementOnDriver("create table Tstage (a int, b int)");
 
-    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVETESTMODEROLLBACKTXN, true);
+    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVE_TEST_MODE_ROLLBACK_TXN, true);
     runStatementOnDriver("insert into Tstage" + TestTxnCommands2.makeValuesClause(dataAbort));
-    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVETESTMODEROLLBACKTXN, false);
+    HiveConf.setBoolVar(hiveConf, HiveConf.ConfVars.HIVE_TEST_MODE_ROLLBACK_TXN, false);
     runStatementOnDriver("insert into Tstage" + TestTxnCommands2.makeValuesClause(data));
     runStatementOnDriver("export table Tstage to '" + getWarehouseDir() + "/1'");
 
@@ -550,4 +597,41 @@ target/tmp/org.apache.hadoop.hive.ql.TestTxnCommands-1521148657811/
         TestTxnCommands2.stringifyValues(data), rs);
 
   }
-}
+
+  @Test
+  public void testImportOrc() throws Exception {
+    // Clear and Drop T and Tstage if exist
+    runStatementOnDriver("drop table if exists T");
+    runStatementOnDriver("drop table if exists Tstage");
+
+    // Create source table - Tstage
+    runStatementOnDriver("create table Tstage (a int, b int) stored" + " as orc tblproperties('transactional'='true')");
+
+    // This creates an ORC data file with correct schema under table root
+    runStatementOnDriver("insert into Tstage values(1,2),(3,4),(5,6)");
+    final int[][] rows = { { 3 } };
+
+    // Check Tstage statistics
+    List<String> rsTStageProperties = runStatementOnDriver("show tblproperties Tstage");
+    Assert.assertEquals("COLUMN_STATS_ACCURATE of Tstage table", true,
+        rsTStageProperties.contains("COLUMN_STATS_ACCURATE\t{\"BASIC_STATS\":\"true\"}"));
+    Assert.assertEquals("numRows of Tstage table", true, rsTStageProperties.contains("numRows\t3"));
+    Assert.assertEquals("numFiles of Tstage table", true, rsTStageProperties.contains("numFiles\t1"));
+
+    // Now we have an archive Tstage table
+    runStatementOnDriver("export table Tstage to '" + getWarehouseDir() + "/1'");
+
+    // Load T
+    runStatementOnDriver("import table T from '" + getWarehouseDir() + "/1'");
+
+    // Check basic stats in tblproperties T
+    List<String> rsTProperties = runStatementOnDriver("show tblproperties T");
+    Assert.assertEquals("COLUMN_STATS_ACCURATE of T table", false,
+        rsTProperties.contains("COLUMN_STATS_ACCURATE\t{\"BASIC_STATS\":\"true\"}"));
+    Assert.assertEquals("numRows of T table", false, rsTProperties.contains("numRows\t3"));
+
+    // Verify the count(*) output
+    List<String> rs = runStatementOnDriver("select count(*) from T");
+    Assert.assertEquals("Rowcount of imported table", TestTxnCommands2.stringifyValues(rows), rs);
+  }
+}
\ No newline at end of file
