@@ -24,9 +24,10 @@ import java.util.ArrayList;
 import java.util.LinkedList;
 import java.util.List;
 
-import junit.framework.TestCase;
+
 
 import org.apache.hadoop.hive.metastore.Warehouse;
+import org.apache.hadoop.hive.ql.util.NullOrdering;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.apache.hadoop.fs.FSDataInputStream;
@@ -34,7 +35,7 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.ql.CompilationOpContext;
-import org.apache.hadoop.hive.ql.DriverContext;
+import org.apache.hadoop.hive.ql.TaskQueue;
 import org.apache.hadoop.hive.ql.QueryState;
 import org.apache.hadoop.hive.ql.exec.mr.ExecDriver;
 import org.apache.hadoop.hive.ql.exec.mr.MapRedTask;
@@ -63,18 +64,21 @@ import org.apache.hadoop.hive.serde.serdeConstants;
 import org.apache.hadoop.hive.serde2.typeinfo.TypeInfoFactory;
 import org.apache.hadoop.hive.shims.ShimLoader;
 import org.apache.hadoop.mapred.TextInputFormat;
+import static org.junit.Assert.assertEquals;
+import org.junit.Before;
+import org.junit.Test;
 
 /**
  * Mimics the actual query compiler in generating end to end plans and testing
  * them out.
  *
  */
-public class TestExecDriver extends TestCase {
+public class TestExecDriver {
 
   static QueryState queryState;
   static HiveConf conf;
 
-  private static final String tmpdir;
+  private static final String TMPDIR;
   private static final Logger LOG = LoggerFactory.getLogger(TestExecDriver.class);
   private static final Path tmppath;
   private static Hive db;
@@ -85,26 +89,26 @@ public class TestExecDriver extends TestCase {
     try {
       queryState = new QueryState.Builder().withHiveConf(new HiveConf(ExecDriver.class)).build();
       conf = queryState.getConf();
-      conf.setBoolVar(HiveConf.ConfVars.SUBMITVIACHILD, true);
-      conf.setBoolVar(HiveConf.ConfVars.SUBMITLOCALTASKVIACHILD, true);
+      conf.setBoolVar(HiveConf.ConfVars.SUBMIT_VIA_CHILD, true);
+      conf.setBoolVar(HiveConf.ConfVars.SUBMIT_LOCAL_TASK_VIA_CHILD, true);
       conf.setVar(HiveConf.ConfVars.HIVE_AUTHORIZATION_MANAGER,
           "org.apache.hadoop.hive.ql.security.authorization.plugin.sqlstd.SQLStdHiveAuthorizerFactory");
 
       SessionState.start(conf);
 
-      tmpdir = System.getProperty("test.tmp.dir");
-      tmppath = new Path(tmpdir);
+      TMPDIR = System.getProperty("test.tmp.dir");
+      tmppath = new Path(TMPDIR);
 
       fs = FileSystem.get(conf);
       if (fs.exists(tmppath) &&
           !ShimLoader.getHadoopShims().isDirectory(fs.getFileStatus(tmppath))) {
-        throw new RuntimeException(tmpdir + " exists but is not a directory");
+        throw new RuntimeException(TMPDIR + " exists but is not a directory");
       }
 
       if (!fs.exists(tmppath)) {
         if (!fs.mkdirs(tmppath)) {
           throw new RuntimeException("Could not make scratch directory "
-              + tmpdir);
+              + TMPDIR);
         }
       }
       LOG.info("Directory of actual files: " + tmppath);
@@ -142,7 +146,7 @@ public class TestExecDriver extends TestCase {
         db.createTable(src, cols, null, TextInputFormat.class,
             HiveIgnoreKeyTextOutputFormat.class);
         db.loadTable(hadoopDataFile[i], src, LoadFileType.KEEP_EXISTING,
-           true, false, false, false, null, 0, false);
+           true, false, false, true, null, 0, false, false);
         i++;
       }
 
@@ -153,8 +157,8 @@ public class TestExecDriver extends TestCase {
 
   MapredWork mr;
 
-  @Override
-  protected void setUp() {
+  @Before
+  public void setUp() {
     mr = PlanUtils.getMapRedWork();
     ctx = new CompilationOpContext();
   }
@@ -168,18 +172,18 @@ public class TestExecDriver extends TestCase {
     String testFileDir = conf.get("test.data.files");
 
     // inbuilt assumption that the testdir has only one output file.
-    Path di_test = new Path(tmppath, testdir);
-    if (!fs.exists(di_test)) {
-      throw new RuntimeException(tmpdir + File.separator + testdir + " does not exist");
+    Path diTest = new Path(tmppath, testdir);
+    if (!fs.exists(diTest)) {
+      throw new RuntimeException(TMPDIR + File.separator + testdir + " does not exist");
     }
-    if (!ShimLoader.getHadoopShims().isDirectory(fs.getFileStatus(di_test))) {
-      throw new RuntimeException(tmpdir + File.separator + testdir + " is not a directory");
+    if (!ShimLoader.getHadoopShims().isDirectory(fs.getFileStatus(diTest))) {
+      throw new RuntimeException(TMPDIR + File.separator + testdir + " is not a directory");
     }
-    FSDataInputStream fi_test = fs.open((fs.listStatus(di_test))[0].getPath());
+    FSDataInputStream fiTest = fs.open((fs.listStatus(diTest))[0].getPath());
 
-    FileInputStream fi_gold = new FileInputStream(new File(testFileDir,datafile));
-    if (!Utilities.contentsEqual(fi_gold, fi_test, false)) {
-      LOG.error(di_test.toString() + " does not match " + datafile);
+    FileInputStream fiGold = new FileInputStream(new File(testFileDir, datafile));
+    if (!Utilities.contentsEqual(fiGold, fiTest, false)) {
+      LOG.error(diTest.toString() + " does not match " + datafile);
       assertEquals(false, true);
     }
   }
@@ -213,7 +217,7 @@ public class TestExecDriver extends TestCase {
   @SuppressWarnings("unchecked")
   private void populateMapPlan1(Table src) throws Exception {
 
-    Operator<FileSinkDesc> op2 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op2 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapplan1.out"), Utilities.defaultTd, true));
     Operator<FilterDesc> op1 = OperatorFactory.get(getTestFilterDesc("key"), op2);
 
@@ -223,7 +227,7 @@ public class TestExecDriver extends TestCase {
   @SuppressWarnings("unchecked")
   private void populateMapPlan2(Table src) throws Exception {
 
-    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapplan2.out"), Utilities.defaultTd, false));
 
     Operator<ScriptDesc> op2 = OperatorFactory.get(new ScriptDesc("cat",
@@ -249,7 +253,7 @@ public class TestExecDriver extends TestCase {
     Operator<ReduceSinkDesc> op1 = OperatorFactory.get(ctx, PlanUtils
         .getReduceSinkDesc(Utilities.makeList(getStringColumn("key")),
         Utilities.makeList(getStringColumn("value")), outputColumns, true,
-        -1, 1, -1, AcidUtils.Operation.NOT_ACID));
+        -1, 1, -1, AcidUtils.Operation.NOT_ACID, NullOrdering.NULLS_LAST));
 
     addMapWork(mr, src, "a", op1);
     ReduceWork rWork = new ReduceWork();
@@ -259,7 +263,7 @@ public class TestExecDriver extends TestCase {
     mr.setReduceWork(rWork);
 
     // reduce side work
-    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapredplan1.out"), Utilities.defaultTd, false));
 
     List<ExprNodeDesc> cols = new ArrayList<ExprNodeDesc>();
@@ -282,7 +286,7 @@ public class TestExecDriver extends TestCase {
         .getReduceSinkDesc(Utilities.makeList(getStringColumn("key")),
         Utilities
         .makeList(getStringColumn("key"), getStringColumn("value")),
-        outputColumns, false, -1, 1, -1, AcidUtils.Operation.NOT_ACID));
+        outputColumns, false, -1, 1, -1, AcidUtils.Operation.NOT_ACID, NullOrdering.NULLS_LAST));
 
     addMapWork(mr, src, "a", op1);
     ReduceWork rWork = new ReduceWork();
@@ -292,7 +296,7 @@ public class TestExecDriver extends TestCase {
     mr.setReduceWork(rWork);
 
     // reduce side work
-    Operator<FileSinkDesc> op4 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op4 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapredplan2.out"), Utilities.defaultTd, false));
 
     Operator<FilterDesc> op3 = OperatorFactory.get(getTestFilterDesc("0"), op4);
@@ -318,14 +322,14 @@ public class TestExecDriver extends TestCase {
     Operator<ReduceSinkDesc> op1 = OperatorFactory.get(ctx, PlanUtils
         .getReduceSinkDesc(Utilities.makeList(getStringColumn("key")),
         Utilities.makeList(getStringColumn("value")), outputColumns, true,
-        Byte.valueOf((byte) 0), 1, -1, AcidUtils.Operation.NOT_ACID));
+        Byte.valueOf((byte) 0), 1, -1, AcidUtils.Operation.NOT_ACID, NullOrdering.NULLS_LAST));
 
     addMapWork(mr, src, "a", op1);
 
     Operator<ReduceSinkDesc> op2 = OperatorFactory.get(ctx, PlanUtils
         .getReduceSinkDesc(Utilities.makeList(getStringColumn("key")),
         Utilities.makeList(getStringColumn("key")), outputColumns, true,
-        Byte.valueOf((byte) 1), Integer.MAX_VALUE, -1, AcidUtils.Operation.NOT_ACID));
+        Byte.valueOf((byte) 1), Integer.MAX_VALUE, -1, AcidUtils.Operation.NOT_ACID, NullOrdering.NULLS_LAST));
 
     addMapWork(mr, src2, "b", op2);
     ReduceWork rWork = new ReduceWork();
@@ -338,7 +342,7 @@ public class TestExecDriver extends TestCase {
     rWork.getTagToValueDesc().add(op2.getConf().getValueSerializeInfo());
 
     // reduce side work
-    Operator<FileSinkDesc> op4 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op4 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapredplan3.out"), Utilities.defaultTd, false));
 
     Operator<SelectDesc> op5 = OperatorFactory.get(new SelectDesc(Utilities
@@ -361,7 +365,8 @@ public class TestExecDriver extends TestCase {
     Operator<ReduceSinkDesc> op1 = OperatorFactory.get(ctx, PlanUtils
         .getReduceSinkDesc(Utilities.makeList(getStringColumn("tkey")),
         Utilities.makeList(getStringColumn("tkey"),
-        getStringColumn("tvalue")), outputColumns, false, -1, 1, -1, AcidUtils.Operation.NOT_ACID));
+        getStringColumn("tvalue")), outputColumns, false, -1, 1, -1,
+                AcidUtils.Operation.NOT_ACID, NullOrdering.NULLS_LAST));
 
     Operator<ScriptDesc> op0 = OperatorFactory.get(new ScriptDesc("cat",
         PlanUtils.getDefaultTableDesc("" + Utilities.tabCode, "key,value"),
@@ -381,7 +386,7 @@ public class TestExecDriver extends TestCase {
     mr.setReduceWork(rWork);
 
     // reduce side work
-    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapredplan4.out"), Utilities.defaultTd, false));
     List<ExprNodeDesc> cols = new ArrayList<ExprNodeDesc>();
     cols.add(getStringColumn(Utilities.ReduceField.KEY + ".reducesinkkey" + 0));
@@ -406,7 +411,7 @@ public class TestExecDriver extends TestCase {
     Operator<ReduceSinkDesc> op0 = OperatorFactory.get(ctx, PlanUtils
         .getReduceSinkDesc(Utilities.makeList(getStringColumn("0")), Utilities
         .makeList(getStringColumn("0"), getStringColumn("1")),
-        outputColumns, false, -1, 1, -1, AcidUtils.Operation.NOT_ACID));
+        outputColumns, false, -1, 1, -1, AcidUtils.Operation.NOT_ACID, NullOrdering.NULLS_LAST));
 
     Operator<SelectDesc> op4 = OperatorFactory.get(new SelectDesc(Utilities
         .makeList(getStringColumn("key"), getStringColumn("value")),
@@ -420,7 +425,7 @@ public class TestExecDriver extends TestCase {
     rWork.getTagToValueDesc().add(op0.getConf().getValueSerializeInfo());
 
     // reduce side work
-    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapredplan5.out"), Utilities.defaultTd, false));
 
     List<ExprNodeDesc> cols = new ArrayList<ExprNodeDesc>();
@@ -441,7 +446,8 @@ public class TestExecDriver extends TestCase {
     Operator<ReduceSinkDesc> op1 = OperatorFactory.get(ctx, PlanUtils
         .getReduceSinkDesc(Utilities.makeList(getStringColumn("tkey")),
         Utilities.makeList(getStringColumn("tkey"),
-        getStringColumn("tvalue")), outputColumns, false, -1, 1, -1, AcidUtils.Operation.NOT_ACID));
+        getStringColumn("tvalue")), outputColumns, false, -1, 1, -1,
+                AcidUtils.Operation.NOT_ACID, NullOrdering.NULLS_LAST));
 
     Operator<ScriptDesc> op0 = OperatorFactory.get(new ScriptDesc(
         "\'cat\'", PlanUtils.getDefaultTableDesc("" + Utilities.tabCode,
@@ -462,7 +468,7 @@ public class TestExecDriver extends TestCase {
     rWork.getTagToValueDesc().add(op1.getConf().getValueSerializeInfo());
 
     // reduce side work
-    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(tmpdir + File.separator
+    Operator<FileSinkDesc> op3 = OperatorFactory.get(ctx, new FileSinkDesc(new Path(TMPDIR + File.separator
         + "mapredplan6.out"), Utilities.defaultTd, false));
 
     Operator<FilterDesc> op2 = OperatorFactory.get(getTestFilterDesc("0"), op3);
@@ -478,10 +484,10 @@ public class TestExecDriver extends TestCase {
   private void executePlan() throws Exception {
     String testName = new Exception().getStackTrace()[1].getMethodName();
     MapRedTask mrtask = new MapRedTask();
-    DriverContext dctx = new DriverContext ();
+    TaskQueue taskQueue = new TaskQueue();
     mrtask.setWork(mr);
-    mrtask.initialize(queryState, null, dctx, null);
-    int exitVal =  mrtask.execute(dctx);
+    mrtask.initialize(queryState, null, taskQueue, null);
+    int exitVal = mrtask.execute();
 
     if (exitVal != 0) {
       LOG.error(testName + " execution failed with exit status: "
@@ -491,6 +497,7 @@ public class TestExecDriver extends TestCase {
     LOG.info(testName + " execution completed successfully");
   }
 
+  @Test
   public void testMapPlan1() throws Exception {
 
     LOG.info("Beginning testMapPlan1");
@@ -499,6 +506,7 @@ public class TestExecDriver extends TestCase {
     fileDiff("lt100.txt.deflate", "mapplan1.out");
   }
 
+  @Test
   public void testMapPlan2() throws Exception {
 
     LOG.info("Beginning testMapPlan2");
@@ -507,6 +515,7 @@ public class TestExecDriver extends TestCase {
     fileDiff("lt100.txt", "mapplan2.out");
   }
 
+  @Test
   public void testMapRedPlan1() throws Exception {
 
     LOG.info("Beginning testMapRedPlan1");
@@ -516,6 +525,7 @@ public class TestExecDriver extends TestCase {
     fileDiff("kv1.val.sorted.txt", "mapredplan1.out");
   }
 
+  @Test
   public void testMapRedPlan2() throws Exception {
 
     LOG.info("Beginning testMapPlan2");
@@ -525,6 +535,7 @@ public class TestExecDriver extends TestCase {
     fileDiff("lt100.sorted.txt", "mapredplan2.out");
   }
 
+  @Test
   public void testMapRedPlan3() throws Exception {
 
     LOG.info("Beginning testMapPlan3");
@@ -534,6 +545,7 @@ public class TestExecDriver extends TestCase {
     fileDiff("kv1kv2.cogroup.txt", "mapredplan3.out");
   }
 
+  @Test
   public void testMapRedPlan4() throws Exception {
 
     LOG.info("Beginning testMapPlan4");
@@ -543,6 +555,7 @@ public class TestExecDriver extends TestCase {
     fileDiff("kv1.string-sorted.txt", "mapredplan4.out");
   }
 
+  @Test
   public void testMapRedPlan5() throws Exception {
 
     LOG.info("Beginning testMapPlan5");
@@ -552,6 +565,7 @@ public class TestExecDriver extends TestCase {
     fileDiff("kv1.string-sorted.txt", "mapredplan5.out");
   }
 
+  @Test
   public void testMapRedPlan6() throws Exception {
 
     LOG.info("Beginning testMapPlan6");
