@@ -20,32 +20,53 @@ package org.apache.hadoop.hive.ql.metadata;
 
 import static org.apache.hadoop.hive.metastore.Warehouse.DEFAULT_DATABASE_NAME;
 
+import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
+import java.util.concurrent.atomic.AtomicReference;
 import java.util.regex.Pattern;
 
+import org.apache.commons.lang3.tuple.Pair;
+import org.apache.hadoop.fs.FileChecksum;
 import org.apache.hadoop.fs.FileStatus;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.conf.HiveConf.ConfVars;
+import org.apache.hadoop.hive.metastore.MetaStoreEventListener;
 import org.apache.hadoop.hive.metastore.PartitionDropOptions;
 import org.apache.hadoop.hive.metastore.Warehouse;
 import org.apache.hadoop.hive.metastore.api.Database;
 import org.apache.hadoop.hive.metastore.api.FieldSchema;
 import org.apache.hadoop.hive.metastore.api.MetaException;
+import org.apache.hadoop.hive.metastore.api.WMNullableResourcePlan;
+import org.apache.hadoop.hive.metastore.api.WMPool;
+import org.apache.hadoop.hive.metastore.api.WMResourcePlan;
+import org.apache.hadoop.hive.metastore.api.WMResourcePlanStatus;
 import org.apache.hadoop.hive.metastore.api.hive_metastoreConstants;
+import org.apache.hadoop.hive.metastore.conf.MetastoreConf;
+import org.apache.hadoop.hive.metastore.events.InsertEvent;
+import org.apache.hadoop.hive.ql.ddl.table.partition.PartitionUtils;
+import org.apache.hadoop.hive.ql.exec.SerializationUtilities;
 import org.apache.hadoop.hive.ql.io.HiveIgnoreKeyTextOutputFormat;
+import org.apache.hadoop.hive.ql.plan.ExprNodeColumnDesc;
+import org.apache.hadoop.hive.ql.plan.ExprNodeConstantDesc;
+import org.apache.hadoop.hive.ql.plan.ExprNodeDesc;
+import org.apache.hadoop.hive.ql.plan.ExprNodeGenericFuncDesc;
 import org.apache.hadoop.hive.ql.session.SessionState;
 import org.apache.hadoop.hive.ql.stats.StatsUtils;
+import org.apache.hadoop.hive.ql.udf.generic.GenericUDFOPAnd;
 import org.apache.hadoop.hive.serde.serdeConstants;
 import org.apache.hadoop.hive.serde2.lazy.LazySimpleSerDe;
 import org.apache.hadoop.hive.serde2.thrift.ThriftDeserializer;
 import org.apache.hadoop.hive.serde2.thrift.test.Complex;
+import org.apache.hadoop.hive.serde2.typeinfo.TypeInfoFactory;
 import org.apache.hadoop.hive.shims.ShimLoader;
 import org.apache.hadoop.mapred.SequenceFileInputFormat;
 import org.apache.hadoop.mapred.SequenceFileOutputFormat;
@@ -59,45 +80,59 @@ import org.apache.logging.log4j.core.config.Configuration;
 import org.apache.logging.log4j.core.config.LoggerConfig;
 import org.apache.thrift.protocol.TBinaryProtocol;
 import org.junit.Assert;
-
 import com.google.common.collect.ImmutableMap;
+import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 
-import junit.framework.TestCase;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.fail;
+import org.junit.BeforeClass;
+import org.junit.AfterClass;
+import org.junit.Test;
 
 /**
  * TestHive.
  *
  */
-public class TestHive extends TestCase {
-  protected Hive hm;
-  protected HiveConf hiveConf;
+public class TestHive {
+  protected static Hive hm;
+  protected static HiveConf hiveConf;
 
-  @Override
-  protected void setUp() throws Exception {
-    super.setUp();
-    hiveConf = new HiveConf(this.getClass());
-    hiveConf
-    .setVar(HiveConf.ConfVars.HIVE_AUTHORIZATION_MANAGER,
+  @BeforeClass
+  public static void setUp() throws Exception {
+
+    hiveConf = new HiveConf(TestHive.class);
+    hm = setUpImpl(hiveConf);
+  }
+
+  private static Hive setUpImpl(HiveConf hiveConf) throws Exception {
+    hiveConf.setVar(HiveConf.ConfVars.HIVE_AUTHORIZATION_MANAGER,
         "org.apache.hadoop.hive.ql.security.authorization.plugin.sqlstd.SQLStdHiveAuthorizerFactory");
     // enable trash so it can be tested
     hiveConf.setFloat("fs.trash.checkpoint.interval", 30);  // FS_TRASH_CHECKPOINT_INTERVAL_KEY (hadoop-2)
     hiveConf.setFloat("fs.trash.interval", 30);             // FS_TRASH_INTERVAL_KEY (hadoop-2)
+    hiveConf.setBoolVar(ConfVars.HIVE_IN_TEST, true);
+    hiveConf.setBoolVar(HiveConf.ConfVars.FIRE_EVENTS_FOR_DML, true);
+    MetastoreConf.setVar(hiveConf, MetastoreConf.ConfVars.EVENT_LISTENERS, DummyFireInsertListener.class.getName());
+    MetastoreConf.setBoolVar(hiveConf, MetastoreConf.ConfVars.HIVE_IN_TEST, true);
     SessionState.start(hiveConf);
     try {
-      hm = Hive.get(hiveConf);
+      return Hive.get(hiveConf);
     } catch (Exception e) {
       System.err.println(StringUtils.stringifyException(e));
-      System.err
-          .println("Unable to initialize Hive Metastore using configuration: \n "
-          + hiveConf);
+      System.err.println("Unable to initialize Hive Metastore using configuration: \n" + hiveConf);
       throw e;
     }
   }
 
-  @Override
-  protected void tearDown() throws Exception {
+  @AfterClass
+  public static void tearDown() throws Exception {
     try {
-      super.tearDown();
+
       // disable trash
       hiveConf.setFloat("fs.trash.checkpoint.interval", 30);  // FS_TRASH_CHECKPOINT_INTERVAL_KEY (hadoop-2)
       hiveConf.setFloat("fs.trash.interval", 30);             // FS_TRASH_INTERVAL_KEY (hadoop-2)
@@ -111,6 +146,7 @@ public class TestHive extends TestCase {
     }
   }
 
+  @Test
   public void testTable() throws Throwable {
     try {
       // create a simple table and test create, drop, get
@@ -210,6 +246,7 @@ public class TestHive extends TestCase {
    *
    * @throws Throwable
    */
+  @Test
   public void testThriftTable() throws Throwable {
     String tableName = "table_for_test_thrifttable";
     try {
@@ -253,6 +290,7 @@ public class TestHive extends TestCase {
    *
    * @throws Throwable
    */
+  @Test
   public void testMetaStoreApiTiming() throws Throwable {
     // Get the RootLogger which, if you don't have log4j2-test.properties defined, will only log ERRORs
     Logger logger = LogManager.getLogger("hive.ql.metadata.Hive");
@@ -307,6 +345,9 @@ public class TestHive extends TestCase {
       tbl.getTTable().setPrivilegesIsSet(false);
 
       ft = hm.getTable(Warehouse.DEFAULT_DATABASE_NAME, tableName);
+      Assert.assertTrue(ft.getTTable().isSetId());
+      ft.getTTable().unsetId();
+
       assertNotNull("Unable to fetch table", ft);
       ft.checkValidity(hiveConf);
       assertEquals("Table names didn't match for table: " + tableName, tbl
@@ -324,6 +365,19 @@ public class TestHive extends TestCase {
       tbl.setCreateTime(ft.getTTable().getCreateTime());
       tbl.getParameters().put(hive_metastoreConstants.DDL_TIME,
           ft.getParameters().get(hive_metastoreConstants.DDL_TIME));
+      // Txn stuff set by metastore
+      if (tbl.getTTable().isSetWriteId() != ft.getTTable().isSetWriteId()) {
+        // No need to compare this field.
+        ft.getTTable().setWriteId(0);
+        tbl.getTTable().setWriteId(0);
+      }
+      // accessType set by HMS Transformer
+      if (tbl.getTTable().isSetAccessType() != ft.getTTable().isSetAccessType()) {
+        // No need to compare this field.
+        tbl.getTTable().setAccessType(ft.getTTable().getAccessType());
+      }
+
+      tbl.getTTable().unsetId();
       assertTrue("Tables  doesn't match: " + tableName + " (" + ft.getTTable()
           + "; " + tbl.getTTable() + ")", ft.getTTable().equals(tbl.getTTable()));
       assertEquals("SerializationLib is not set correctly", tbl
@@ -351,6 +405,7 @@ public class TestHive extends TestCase {
    * Test basic Hive class interaction, that:
    * - We can have different Hive objects throughout the lifetime of this thread.
    */
+  @Test
   public void testHiveCloseCurrent() throws Throwable {
     Hive hive1 = Hive.get();
     Hive.closeCurrent();
@@ -359,6 +414,7 @@ public class TestHive extends TestCase {
     assertTrue(hive1 != hive2);
   }
 
+  @Test
   public void testGetAndDropTables() throws Throwable {
     try {
       String dbName = "db_for_testgettables";
@@ -411,6 +467,54 @@ public class TestHive extends TestCase {
     }
   }
 
+  @Test
+  public void testWmNamespaceHandling() throws Throwable {
+    HiveConf hiveConf = new HiveConf(this.getClass());
+    Hive hm = setUpImpl(hiveConf);
+    // TODO: threadlocals... Why is all this Hive client stuff like that?!!
+    final AtomicReference<Hive> hm2r = new AtomicReference<>();
+    Thread pointlessThread = new Thread(new Runnable() {
+      @Override
+      public void run() {
+        HiveConf hiveConf2 = new HiveConf(this.getClass());
+        hiveConf2.setVar(ConfVars.HIVE_SERVER2_WM_NAMESPACE, "hm2");
+        try {
+          hm2r.set(setUpImpl(hiveConf2));
+        } catch (Exception e) {
+          System.err.println(StringUtils.stringifyException(e));
+        }
+      }
+    });
+    pointlessThread.start();
+    pointlessThread.join();
+    Hive hm2 = hm2r.get();
+    assertNotNull(hm2);
+
+    hm.createResourcePlan(new WMResourcePlan("hm"), null, false);
+    assertEquals(1, hm.getAllResourcePlans().size());
+    assertEquals(0, hm2.getAllResourcePlans().size());
+    hm2.createResourcePlan(new WMResourcePlan("hm"), null, false);
+    WMNullableResourcePlan changes = new WMNullableResourcePlan();
+    changes.setStatus(WMResourcePlanStatus.ACTIVE);
+    hm.alterResourcePlan("hm", changes, true, false, false);
+    // We should not be able to modify the active plan.
+    WMPool pool = new WMPool("hm", "foo");
+    pool.setAllocFraction(0);
+    pool.setQueryParallelism(1);
+    try {
+      hm.createWMPool(pool);
+      fail("Expected exception");
+    } catch (HiveException e) {
+    }
+    // But we should still be able to modify the other plan.
+    pool.unsetNs(); // The call to create sets the namespace.
+    hm2.createWMPool(pool);
+    // Make the 2nd plan active in a different namespace.
+    changes.unsetNs();
+    hm2.alterResourcePlan("hm", changes, true, false, false);
+  }
+
+  @Test
   public void testDropTableTrash() throws Throwable {
     if (!ShimLoader.getHadoopShims().supportTrashFeature()) {
       return; // it's hadoop-1
@@ -523,6 +627,7 @@ public class TestHive extends TestCase {
    * 2. Drop partitions with PURGE, and check that the data is moved to Trash.
    * @throws Exception on failure.
    */
+  @Test
   public void testDropPartitionsWithPurge() throws Exception {
     String dbName = Warehouse.DEFAULT_DATABASE_NAME;
     String tableName = "table_for_testDropPartitionsWithPurge";
@@ -581,10 +686,50 @@ public class TestHive extends TestCase {
     }
   }
 
+  @Test
+  public void testDropMissingPartitionsByFilter() throws Throwable {
+    String dbName = Warehouse.DEFAULT_DATABASE_NAME;
+    String tableName = "table_for_testDropMissingPartitionsByFilter";
+
+    Table table = createPartitionedTable(dbName, tableName);
+    for (int i = 10; i <= 12; i++) {
+      Map<String, String> partitionSpec = new ImmutableMap.Builder<String, String>()
+          .put("ds", "20231129")
+          .put("hr", String.valueOf(i))
+          .build();
+      hm.createPartition(table, partitionSpec);
+    }
+
+    List<Partition> partitions = hm.getPartitions(table);
+    assertEquals(3, partitions.size());
+
+    // drop partitions by filter with missing predicate
+    try {
+      List<Pair<Integer, byte[]>> partExprs = new ArrayList<>();
+      ExprNodeColumnDesc column = new ExprNodeColumnDesc(
+          TypeInfoFactory.stringTypeInfo, "ds", null, true);
+      List<String> values = Arrays.asList("20231130", "20231129");
+      for (int i = 0; i < values.size(); i++) {
+        ExprNodeGenericFuncDesc expr = PartitionUtils.makeBinaryPredicate(
+            "=", column, new ExprNodeConstantDesc(TypeInfoFactory.stringTypeInfo, values.get(i)));
+        partExprs.add(Pair.of(i, SerializationUtilities.serializeObjectWithTypeInformation(expr)));
+      }
+      hm.dropPartitions(dbName, tableName, partExprs, PartitionDropOptions.instance());
+      fail("Expected exception");
+    } catch (HiveException e) {
+      // expected
+      assertEquals("Some partitions to drop are missing", e.getCause().getMessage());
+      assertEquals(3, hm.getPartitions(table).size());
+    } finally {
+      cleanUpTableQuietly(dbName, tableName);
+    }
+  }
+
   /**
    * Test that tables set up with auto-purge skip trash-directory when tables/partitions are dropped.
    * @throws Throwable
    */
+  @Test
   public void testAutoPurgeTablesAndPartitions() throws Throwable {
 
     String dbName = Warehouse.DEFAULT_DATABASE_NAME;
@@ -592,8 +737,8 @@ public class TestHive extends TestCase {
     try {
 
       Table table = createPartitionedTable(dbName, tableName);
-      table.getParameters().put("auto.purge", "true");
-      hm.alterTable(tableName, table, null);
+      table.getParameters().put("skip.trash", "true");
+      hm.alterTable(tableName, table, false, null, true);
 
       Map<String, String> partitionSpec =  new ImmutableMap.Builder<String, String>()
           .put("ds", "20141216")
@@ -637,6 +782,7 @@ public class TestHive extends TestCase {
     }
   }
 
+  @Test
   public void testPartition() throws Throwable {
     try {
       String tableName = "table_for_testpartition";
@@ -714,6 +860,7 @@ public class TestHive extends TestCase {
         System.err.println(StringUtils.stringifyException(e));
         assertTrue("Unable to create parition for table: " + tableName, false);
       }
+      checkPartitionsConsistency(tbl);
 
       Map<String, String> partialSpec = new HashMap<>();
       partialSpec.put("ds", "2008-04-07");
@@ -734,6 +881,9 @@ public class TestHive extends TestCase {
       partialSpec.put("hr", "14");
       assertEquals(1, hm.getPartitions(tbl, partialSpec).size());
 
+      // Test get partitions with max_parts
+      assertEquals(1, hm.getPartitions(tbl, new HashMap(), (short) 1).size());
+
       hm.dropTable(Warehouse.DEFAULT_DATABASE_NAME, tableName);
     } catch (Throwable e) {
       System.err.println(StringUtils.stringifyException(e));
@@ -742,6 +892,81 @@ public class TestHive extends TestCase {
     }
   }
 
+  @Test
+  public void testGetPartitionsWithMaxLimit() throws Exception {
+    String dbName = Warehouse.DEFAULT_DATABASE_NAME;
+    String tableName = "table_for_get_partitions_with_max_limit";
+
+    try {
+      Map<String, String> part_spec = new HashMap<String, String>();
+
+      Table table = createPartitionedTable(dbName, tableName);
+      part_spec.clear();
+      part_spec.put("ds", "2025-06-30");
+      part_spec.put("hr", "11");
+      hm.createPartition(table, part_spec);
+
+      Thread.sleep(1);
+      part_spec.clear();
+      part_spec.put("ds", "2023-04-15");
+      part_spec.put("hr", "12");
+      hm.createPartition(table, part_spec);
+
+      Thread.sleep(1);
+      part_spec.clear();
+      part_spec.put("ds", "2023-09-01");
+      part_spec.put("hr", "10");
+      hm.createPartition(table, part_spec);
+
+      // Default
+      Assert.assertEquals(
+          ((List<Partition>) hm.getPartitions(table, new HashMap(), (short) 1)).get(0).getTPartition().getValues(),
+          Arrays.asList("2023-04-15", "12"));
+
+      // Sort by "PARTITIONS"."CREATE_TIME" desc
+      hm.setMetaConf(MetastoreConf.ConfVars.PARTITION_ORDER_EXPR.getVarname(), "\"PARTITIONS\".\"CREATE_TIME\" desc");
+      Assert.assertEquals(
+          ((List<Partition>) hm.getPartitions(table, new HashMap(), (short) 1)).get(0).getTPartition().getValues(),
+          Arrays.asList("2023-09-01", "10"));
+
+      // Sort by "PART_NAME" desc
+      hm.setMetaConf(MetastoreConf.ConfVars.PARTITION_ORDER_EXPR.getVarname(), "\"PART_NAME\" desc");
+      Assert.assertEquals(
+          ((List<Partition>) hm.getPartitions(table, new HashMap(), (short) 1)).get(0).getTPartition().getValues(),
+          Arrays.asList("2025-06-30", "11"));
+
+      // Test MetaStoreClient
+      Assert.assertEquals(
+          hm.getMSC().listPartitions(table.getDbName(), table.getTableName(), (short) 1).get(0).getValues(),
+          Arrays.asList("2025-06-30", "11"));
+    } catch (Exception e) {
+      fail("Unexpected exception: " + StringUtils.stringifyException(e));
+    } finally {
+      hm.setMetaConf(MetastoreConf.ConfVars.PARTITION_ORDER_EXPR.getVarname(),
+         MetastoreConf.ConfVars.PARTITION_ORDER_EXPR.getDefaultVal().toString());
+      cleanUpTableQuietly(dbName, tableName);
+    }
+  }
+
+  private void checkPartitionsConsistency(Table tbl) throws Exception {
+    Set<Partition> allParts = hm.getAllPartitionsOf(tbl);
+    List<Partition> allParts2 = hm.getPartitions(tbl);
+    assertEquals("inconsistent results: getAllPartitionsOf/getPartitions", allParts, new HashSet<>(allParts2));
+
+    Partition singlePart = allParts2.get(0);
+    Partition singlePart2 = hm.getPartition(tbl, singlePart.getSpec(), false);
+    assertEquals("inconsistent results: getPartition", singlePart, singlePart2);
+
+    List<ExprNodeDesc> exprs = Lists.newArrayList(new ExprNodeConstantDesc(true), new ExprNodeConstantDesc(true));
+    ExprNodeGenericFuncDesc trueExpr = new ExprNodeGenericFuncDesc(TypeInfoFactory.booleanTypeInfo, new GenericUDFOPAnd(), "and", exprs);
+    List<Partition> allParts3 = new ArrayList<Partition>();
+    hm.getPartitionsByExpr(tbl, trueExpr, hm.getConf(), allParts3);
+
+    assertEquals("inconsistent results: getPartitionsByExpr", allParts2, allParts3);
+
+  }
+
+  @Test
   public void testHiveRefreshOnConfChange() throws Throwable{
     Hive prevHiveObj = Hive.get();
     prevHiveObj.getDatabaseCurrent();
@@ -761,12 +986,75 @@ public class TestHive extends TestCase {
     prevHiveObj.getDatabaseCurrent();
     //change value of a metavar config param in new hive conf
     newHconf = new HiveConf(hiveConf);
-    newHconf.setIntVar(ConfVars.METASTORETHRIFTCONNECTIONRETRIES,
-        newHconf.getIntVar(ConfVars.METASTORETHRIFTCONNECTIONRETRIES) + 1);
+    newHconf.setIntVar(ConfVars.METASTORE_THRIFT_CONNECTION_RETRIES,
+        newHconf.getIntVar(ConfVars.METASTORE_THRIFT_CONNECTION_RETRIES) + 1);
     newHiveObj = Hive.get(newHconf);
     assertTrue(prevHiveObj != newHiveObj);
   }
 
+  public void testFireInsertEvent() throws Throwable {
+    Hive hiveDb = Hive.getWithFastCheck(hiveConf, false);
+    String tableName = "test_fire_insert_event";
+    hiveDb.dropTable(tableName);
+    hiveDb.createTable(tableName, Lists.newArrayList("col1"), null, TextInputFormat.class,
+        HiveIgnoreKeyTextOutputFormat.class);
+    Table table = hiveDb.getTable(tableName);
+    Path tablePath = table.getDataLocation();
+    // Create some files that "inserted"
+    FileSystem fileSystem = tablePath.getFileSystem(hiveConf);
+    fileSystem.deleteOnExit(tablePath);
+    Path insert1 = new Path(tablePath, "insert1"), insert2 = new Path(tablePath, "insert2"),
+        insert3 = new Path(tablePath, "insert3");
+
+    try (OutputStream os1 = fileSystem.create(insert1);
+         OutputStream os2 = fileSystem.create(insert2);
+         OutputStream os3 = fileSystem.create(insert3)) {
+      os1.write(new StringBuilder("hello, ").append(System.lineSeparator())
+          .append("world1").toString().getBytes());
+      os2.write(new StringBuilder("hello, ").append(System.lineSeparator())
+          .append("world2").toString().getBytes());
+      os3.write(new StringBuilder("hello, ").append(System.lineSeparator())
+          .append("world3").toString().getBytes());
+    }
+
+    // Fire the InsertData event
+    hiveDb.fireInsertEvent(hiveDb.getDatabaseCurrent().getName(), tableName, null,true,
+        Arrays.asList(insert1.toString(), insert2.toString(), insert3.toString()));
+    // Get the last Metastore event
+    InsertEvent insertEvent = DummyFireInsertListener.getLastEvent();
+    // Check the event
+    Assert.assertNotNull(insertEvent);
+    Assert.assertNotNull(insertEvent.getTableObj());
+    Assert.assertEquals(tableName, insertEvent.getTableObj().getTableName());
+    Assert.assertEquals(hiveDb.getDatabaseCurrent().getName(), insertEvent.getTableObj().getDbName());
+    Set<String> insertFiles = new HashSet<>(insertEvent.getFiles());
+    Set<String> expectedFiles = Sets.newHashSet(insert1.toString(), insert2.toString(), insert3.toString());
+    Assert.assertTrue(insertFiles.size() == 3);
+    for (String insertFile : insertFiles) {
+      Assert.assertTrue(expectedFiles.contains(insertFile));
+    }
+    Map<String, String> expectedCheckSums = new HashMap<>();
+    expectedCheckSums.put("insert1", getFileCheckSum(fileSystem, insert1));
+    expectedCheckSums.put("insert2", getFileCheckSum(fileSystem, insert2));
+    expectedCheckSums.put("insert3", getFileCheckSum(fileSystem, insert3));
+    List<String> checkSums = insertEvent.getFileChecksums();
+    Assert.assertTrue(checkSums.size() == 3);
+    for (int i = 0; i < 3; i++) {
+      Path insertedPath = new Path(insertEvent.getFiles().get(i));
+      Assert.assertEquals(expectedCheckSums.get(insertedPath.getName()), checkSums.get(i));
+    }
+  }
+
+  private String getFileCheckSum(FileSystem fileSystem, Path p) throws Exception {
+    FileChecksum cksum = fileSystem.getFileChecksum(p);
+    if (cksum != null) {
+      String checksumString =
+          StringUtils.byteToHexString(cksum.getBytes(), 0, cksum.getLength());
+      return checksumString;
+    }
+    return "";
+  }
+
   // shamelessly copied from Path in hadoop-2
   private static final String SEPARATOR = "/";
   private static final char SEPARATOR_CHAR = '/';
@@ -801,4 +1089,22 @@ public class TestHive extends TestCase {
   private static boolean hasWindowsDrive(String path) {
     return (WINDOWS && hasDriveLetterSpecifier.matcher(path).find());
   }
+
+  public static class DummyFireInsertListener extends MetaStoreEventListener {
+    private static final List<InsertEvent> notifyList = new ArrayList<>();
+    public DummyFireInsertListener(org.apache.hadoop.conf.Configuration conf) {
+      super(conf);
+    }
+    @Override
+    public void onInsert(InsertEvent insertEvent) throws MetaException {
+      notifyList.add(insertEvent);
+    }
+    public static InsertEvent getLastEvent() {
+      if (notifyList.isEmpty()) {
+        return null;
+      } else {
+        return notifyList.get(notifyList.size() - 1);
+      }
+    }
+  }
 }
