@@ -19,9 +19,9 @@
 package org.apache.hadoop.hive.ql.parse;
 
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 
 import org.junit.Assert;
 
@@ -39,12 +39,10 @@ public class TestColumnAccess {
   @BeforeClass
   public static void Setup() throws Exception {
     Driver driver = createDriver();
-    int ret = driver.run("create table t1(id1 int, name1 string)").getResponseCode();
-    Assert.assertEquals("Checking command success", 0, ret);
-    ret = driver.run("create table t2(id2 int, id1 int, name2 string)").getResponseCode();
-    Assert.assertEquals("Checking command success", 0, ret);
-    ret = driver.run("create view v1 as select * from t1").getResponseCode();
-    Assert.assertEquals("Checking command success", 0, ret);
+    driver.run("create table t1(id1 int, name1 string)");
+    driver.run("create table t2(id2 int, id1 int, name2 string)");
+    driver.run("create table t3(id1 int) partitioned by (`date` int, p0 string)");
+    driver.run("create view v1 as select * from t1");
   }
 
   @AfterClass
@@ -52,6 +50,7 @@ public class TestColumnAccess {
     Driver driver = createDriver();
     driver.run("drop table t1");
     driver.run("drop table t2");
+    driver.run("drop table t3");
     driver.run("drop view v1");
   }
 
@@ -59,7 +58,7 @@ public class TestColumnAccess {
   public void testQueryTable1() throws ParseException {
     String query = "select * from t1";
     Driver driver = createDriver();
-    int rc = driver.compile(query);
+    int rc = driver.compile(query, true);
     Assert.assertEquals("Checking command success", 0, rc);
     QueryPlan plan = driver.getPlan();
     // check access columns from ColumnAccessInfo
@@ -67,23 +66,23 @@ public class TestColumnAccess {
     List<String> cols = columnAccessInfo.getTableToColumnAccessMap().get("default@t1");
     Assert.assertNotNull(cols);
     Assert.assertEquals(2, cols.size());
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name1"));
 
     // check access columns from readEntity
     Map<String, List<String>> tableColsMap = getColsFromReadEntity(plan.getInputs());
     cols = tableColsMap.get("default@t1");
     Assert.assertNotNull(cols);
     Assert.assertEquals(2, cols.size());
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name1"));
   }
 
   @Test
   public void testJoinTable1AndTable2() throws ParseException {
     String query = "select * from t1 join t2 on (t1.id1 = t2.id1)";
     Driver driver = createDriver();
-    int rc = driver.compile(query);
+    int rc = driver.compile(query, true);
     Assert.assertEquals("Checking command success", 0, rc);
     QueryPlan plan = driver.getPlan();
     // check access columns from ColumnAccessInfo
@@ -91,14 +90,14 @@ public class TestColumnAccess {
     List<String> cols = columnAccessInfo.getTableToColumnAccessMap().get("default@t1");
     Assert.assertNotNull(cols);
     Assert.assertEquals(2, cols.size());
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name1"));
     cols = columnAccessInfo.getTableToColumnAccessMap().get("default@t2");
     Assert.assertNotNull(cols);
     Assert.assertEquals(3, cols.size());
-    Assert.assertNotNull(cols.contains("id2"));
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id2"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name2"));
 
 
     // check access columns from readEntity
@@ -106,21 +105,21 @@ public class TestColumnAccess {
     cols = tableColsMap.get("default@t1");
     Assert.assertNotNull(cols);
     Assert.assertEquals(2, cols.size());
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name1"));
     cols = tableColsMap.get("default@t2");
     Assert.assertNotNull(cols);
     Assert.assertEquals(3, cols.size());
-    Assert.assertNotNull(cols.contains("id2"));
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id2"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name2"));
   }
 
   @Test
   public void testJoinView1AndTable2() throws ParseException {
     String query = "select * from v1 join t2 on (v1.id1 = t2.id1)";
     Driver driver = createDriver();
-    int rc = driver.compile(query);
+    int rc = driver.compile(query, true);
     Assert.assertEquals("Checking command success", 0, rc);
     QueryPlan plan = driver.getPlan();
     // check access columns from ColumnAccessInfo
@@ -132,14 +131,14 @@ public class TestColumnAccess {
     cols = columnAccessInfo.getTableToColumnAccessMap().get("default@v1");
     Assert.assertNotNull(cols);
     Assert.assertEquals(2, cols.size());
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name1"));
     cols = columnAccessInfo.getTableToColumnAccessMap().get("default@t2");
     Assert.assertNotNull(cols);
     Assert.assertEquals(3, cols.size());
-    Assert.assertNotNull(cols.contains("id2"));
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id2"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name2"));
 
 
     // check access columns from readEntity
@@ -149,17 +148,32 @@ public class TestColumnAccess {
     cols = tableColsMap.get("default@v1");
     Assert.assertNotNull(cols);
     Assert.assertEquals(2, cols.size());
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name1"));
     cols = tableColsMap.get("default@t2");
     Assert.assertNotNull(cols);
     Assert.assertEquals(3, cols.size());
-    Assert.assertNotNull(cols.contains("id2"));
-    Assert.assertNotNull(cols.contains("id1"));
-    Assert.assertNotNull(cols.contains("name1"));
+    Assert.assertTrue(cols.contains("id2"));
+    Assert.assertTrue(cols.contains("id1"));
+    Assert.assertTrue(cols.contains("name2"));
   }
 
-  private Map<String, List<String>> getColsFromReadEntity(HashSet<ReadEntity> inputs) {
+  @Test
+  public void testShowPartitions() throws Exception {
+    String query = "show partitions t3 where `date` > 20011113";
+    Driver driver = createDriver();
+    int rc = driver.compile(query, true);
+    Assert.assertEquals("Checking command success", 0, rc);
+    QueryPlan plan = driver.getPlan();
+    ColumnAccessInfo columnAccessInfo = plan.getColumnAccessInfo();
+    List<String> cols = columnAccessInfo.getTableToColumnAccessMap().get("default@t3");
+
+    Assert.assertEquals(2, cols.size());
+    Assert.assertTrue(cols.contains("date"));
+    Assert.assertTrue(cols.contains("p0"));
+  }
+
+  private Map<String, List<String>> getColsFromReadEntity(Set<ReadEntity> inputs) {
     Map<String, List<String>> tableColsMap = new HashMap<String, List<String>>();
     for(ReadEntity entity: inputs) {
       switch (entity.getType()) {
