@@ -19,12 +19,12 @@
 package org.apache.hadoop.hive.ql.processors;
 
 import java.io.ByteArrayOutputStream;
-import java.io.PrintStream;
 import java.lang.reflect.Field;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
 
+import org.apache.hadoop.hive.common.io.SessionStream;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.conf.SystemVariables;
 import org.apache.hadoop.hive.ql.session.SessionState;
@@ -55,9 +55,9 @@ public class TestSetProcessor {
   }
 
   @Before
-  public void setupTest() {
+  public void setupTest() throws Exception {
     baos = new ByteArrayOutputStream();
-    state.out = new PrintStream(baos);
+    state.out = new SessionStream(baos);
     processor = new SetProcessor();
   }
 
@@ -65,13 +65,13 @@ public class TestSetProcessor {
   public void testHiddenConfig() throws Exception {
     runSetProcessor("");
     String output = baos.toString();
-    Assert.assertFalse(output.contains(HiveConf.ConfVars.METASTOREPWD.varname + "="));
+    Assert.assertFalse(output.contains(HiveConf.ConfVars.METASTORE_PWD.varname + "="));
     Assert.assertFalse(output.contains(HiveConf.ConfVars.HIVE_SERVER2_SSL_KEYSTORE_PASSWORD.varname + "="));
   }
 
   @Test
-  public void testHiddenConfigSetVarName() {
-    runSetProcessor(HiveConf.ConfVars.METASTOREPWD.varname);
+  public void testHiddenConfigSetVarName() throws CommandProcessorException {
+    runSetProcessor(HiveConf.ConfVars.METASTORE_PWD.varname);
     String output = baos.toString();
     Assert.assertTrue(output.contains("hidden"));
   }
@@ -109,7 +109,7 @@ public class TestSetProcessor {
   /*
    * Simulates the set <command>;
    */
-  private void runSetProcessor(String command) {
+  private void runSetProcessor(String command) throws CommandProcessorException {
     processor.run(command);
     state.out.flush();
   }
