@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -21,23 +21,31 @@ package org.apache.hadoop.hive.serde2.objectinspector;
 import java.lang.reflect.Field;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.Type;
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
-import java.util.HashMap;
+import java.util.Arrays;
 import java.util.Iterator;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.hadoop.hive.serde2.io.DateWritableV2;
+import org.apache.hadoop.hive.serde2.io.TimestampLocalTZWritable;
+import org.apache.hadoop.hive.serde2.io.TimestampWritable;
+import org.apache.hadoop.hive.serde2.io.TimestampWritableV2;
+import org.apache.hadoop.hive.serde2.objectinspector.primitive.SettableTimestampLocalTZObjectInspector;
+import org.apache.hadoop.hive.serde2.objectinspector.primitive.TimestampLocalTZObjectInspector;
+import org.apache.hive.common.util.Murmur3;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.hive.serde.serdeConstants;
 import org.apache.hadoop.hive.serde2.SerDeException;
-import org.apache.hadoop.hive.serde2.io.DateWritable;
 import org.apache.hadoop.hive.serde2.io.HiveCharWritable;
 import org.apache.hadoop.hive.serde2.io.HiveDecimalWritable;
 import org.apache.hadoop.hive.serde2.io.HiveIntervalDayTimeWritable;
 import org.apache.hadoop.hive.serde2.io.HiveIntervalYearMonthWritable;
 import org.apache.hadoop.hive.serde2.io.HiveVarcharWritable;
-import org.apache.hadoop.hive.serde2.io.TimestampWritable;
+import org.apache.hadoop.hive.serde2.lazy.LazyDouble;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspector.Category;
 import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspectorFactory.ObjectInspectorOptions;
 import org.apache.hadoop.hive.serde2.objectinspector.primitive.AbstractPrimitiveWritableObjectInspector;
@@ -77,6 +85,7 @@ import org.apache.hadoop.hive.serde2.objectinspector.primitive.TimestampObjectIn
 import org.apache.hadoop.hive.serde2.objectinspector.primitive.WritableStringObjectInspector;
 import org.apache.hadoop.hive.serde2.typeinfo.TypeInfoUtils;
 import org.apache.hadoop.io.BytesWritable;
+import org.apache.hadoop.io.DoubleWritable;
 import org.apache.hadoop.io.Text;
 import org.apache.hadoop.util.StringUtils;
 
@@ -89,7 +98,7 @@ import org.apache.hadoop.util.StringUtils;
  */
 public final class ObjectInspectorUtils {
 
-  protected final static Log LOG = LogFactory.getLog(ObjectInspectorUtils.class.getName());
+  private static final Logger LOG = LoggerFactory.getLogger(ObjectInspectorUtils.class.getName());
 
   /**
    * This enum controls how we copy primitive objects.
@@ -103,6 +112,94 @@ public final class ObjectInspectorUtils {
     DEFAULT, JAVA, WRITABLE
   }
 
+  /**
+   * This enum controls how we interpret null value when compare two objects.
+   *
+   * MINVALUE means treating null value as the minimum value.
+   * MAXVALUE means treating null value as the maximum value.
+   *
+   */
+  public enum NullValueOption {
+    MINVALUE(-1),
+    MAXVALUE(1);
+
+    private final int cmpReturnValue;
+
+    NullValueOption(int cmpReturnValue) {
+      this.cmpReturnValue = cmpReturnValue;
+    }
+
+    public int getCmpReturnValue() {
+      return cmpReturnValue;
+    }
+  }
+
+  /**
+   * This class can be used to wrap Hive objects and put in HashMap or HashSet.
+   * The objects will be compared using ObjectInspectors.
+   *
+   */
+  public static class ObjectInspectorObject {
+    private final Object[] objects;
+    private final ObjectInspector[] oi;
+
+    public ObjectInspectorObject(Object object, ObjectInspector oi) {
+      this.objects = new Object[] { object };
+      this.oi = new ObjectInspector[] { oi };
+    }
+
+    public ObjectInspectorObject(Object[] objects, ObjectInspector[] oi) {
+      this.objects = objects;
+      this.oi = oi;
+    }
+
+    public Object[] getValues() {
+      return objects;
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+      if (this == obj) return true;
+      if (obj == null || obj.getClass() != this.getClass()) { return false; }
+
+      ObjectInspectorObject comparedObject = (ObjectInspectorObject)obj;
+      return ObjectInspectorUtils.compare(objects, oi, comparedObject.objects, comparedObject.oi) == 0;
+    }
+
+    @Override
+    public int hashCode() {
+      return ObjectInspectorUtils.getBucketHashCodeOld(objects, oi);
+    }
+  }
+
+  /**
+   * Calculates the hash code for array of Objects that contains writables. This is used
+   * to work around the buggy Hadoop DoubleWritable hashCode implementation. This should
+   * only be used for process-local hash codes; don't replace stored hash codes like bucketing.
+   */
+  public static int writableArrayHashCode(Object[] keys) {
+    if (keys == null) return 0;
+    int hashcode = 1;
+    for (Object element : keys) {
+      hashcode = 31 * hashcode;
+      if (element == null) {
+        // nothing
+      } else if (element instanceof LazyDouble) {
+        long v = Double.doubleToLongBits(((LazyDouble) element).getWritableObject().get());
+        hashcode = hashcode + (int) (v ^ (v >>> 32));
+      } else if (element instanceof DoubleWritable) {
+        long v = Double.doubleToLongBits(((DoubleWritable) element).get());
+        hashcode = hashcode + (int) (v ^ (v >>> 32));
+      } else if (element instanceof Object[]) {
+        // use deep hashcode for arrays
+        hashcode = hashcode + Arrays.deepHashCode((Object[]) element);
+      } else {
+        hashcode = hashcode + element.hashCode();
+      }
+    }
+    return hashcode;
+  }
+
   /**
    * Ensures that an ObjectInspector is Writable.
    */
@@ -118,6 +215,21 @@ public final class ObjectInspectorUtils {
     return oi;
   }
 
+  /**
+   * Get the corresponding standard ObjectInspector array for an array of ObjectInspector.
+   */
+  public static ObjectInspector[] getStandardObjectInspector(ObjectInspector[] ois,
+      ObjectInspectorCopyOption objectInspectorOption) {
+    if (ois == null) return null;
+
+    ObjectInspector[] result = new ObjectInspector[ois.length];
+    for (int i = 0; i < ois.length; i++) {
+      result[i] = getStandardObjectInspector(ois[i], objectInspectorOption);
+    }
+
+    return result;
+  }
+
   /**
    * Get the corresponding standard ObjectInspector for an ObjectInspector.
    *
@@ -247,6 +359,23 @@ public final class ObjectInspectorUtils {
     }
   }
 
+  /**
+   * Returns a deep copy of an array of objects
+   */
+  public static Object[] copyToStandardObject(
+      Object[] o, ObjectInspector[] oi, ObjectInspectorCopyOption objectInspectorOption) {
+    if (o == null) return null;
+    assert(o.length == oi.length);
+
+    Object[] result = new Object[o.length];
+    for (int i = 0; i < o.length; i++) {
+      result[i] = ObjectInspectorUtils.copyToStandardObject(
+          o[i], oi[i], objectInspectorOption);
+    }
+
+    return result;
+  }
+
   /**
    * Returns a deep copy of the Object o that can be scanned by a
    * StandardObjectInspector returned by getStandardObjectInspector(oi).
@@ -305,6 +434,10 @@ public final class ObjectInspectorUtils {
         result = loi.getPrimitiveJavaObject(o);
         if (loi.getPrimitiveCategory() == PrimitiveObjectInspector.PrimitiveCategory.TIMESTAMP) {
           result = PrimitiveObjectInspectorFactory.javaTimestampObjectInspector.copyObject(result);
+        } else if (loi.getPrimitiveCategory() ==
+            PrimitiveObjectInspector.PrimitiveCategory.TIMESTAMPLOCALTZ) {
+          result = PrimitiveObjectInspectorFactory.javaTimestampTZObjectInspector.
+              copyObject(result);
         }
         break;
       case WRITABLE:
@@ -326,7 +459,7 @@ public final class ObjectInspectorUtils {
     }
     case MAP: {
       MapObjectInspector moi = (MapObjectInspector) oi;
-      HashMap<Object, Object> map = new HashMap<Object, Object>();
+      Map<Object, Object> map = new LinkedHashMap<Object, Object>();
       Map<? extends Object, ? extends Object> omap = moi.getMap(o);
       for (Map.Entry<? extends Object, ? extends Object> entry : omap
           .entrySet()) {
@@ -352,11 +485,12 @@ public final class ObjectInspectorUtils {
     case UNION: {
       UnionObjectInspector uoi = (UnionObjectInspector)oi;
       List<ObjectInspector> objectInspectors = uoi.getObjectInspectors();
+      byte tag = uoi.getTag(o);
       Object object = copyToStandardObject(
               uoi.getField(o),
-              objectInspectors.get(uoi.getTag(o)),
+              objectInspectors.get(tag),
               objectInspectorOption);
-      result = object;
+      result = new StandardUnionObjectInspector.StandardUnion(tag, object);
       break;
     }
     default: {
@@ -475,7 +609,7 @@ public final class ObjectInspectorUtils {
       return result.toString();
     }
     case UNION: {
-      StringBuffer result = new StringBuffer();
+      StringBuilder result = new StringBuilder();
       result.append(oi.getClass().getSimpleName() + "<");
       UnionObjectInspector uoi = (UnionObjectInspector)oi;
       List<ObjectInspector> ois = uoi.getObjectInspectors();
@@ -494,6 +628,58 @@ public final class ObjectInspectorUtils {
     }
   }
 
+  /**
+   * Computes the bucket number to which the bucketFields belong to
+   * @param bucketFields  the bucketed fields of the row
+   * @param bucketFieldInspectors  the ObjectInspectors for each of the bucketed fields
+   * @param totalBuckets the number of buckets in the table
+   * @return the bucket number using Murmur hash
+   */
+  public static int getBucketNumber(Object[] bucketFields, ObjectInspector[] bucketFieldInspectors, int totalBuckets) {
+    return getBucketNumber(getBucketHashCode(bucketFields, bucketFieldInspectors), totalBuckets);
+  }
+
+  /**
+   * Computes the bucket number to which the bucketFields belong to
+   * @param bucketFields  the bucketed fields of the row
+   * @param bucketFieldInspectors  the ObjectInspectors for each of the bucketed fields
+   * @param totalBuckets the number of buckets in the table
+   * @return the bucket number
+   */
+  @Deprecated
+  public static int getBucketNumberOld(Object[] bucketFields, ObjectInspector[] bucketFieldInspectors, int totalBuckets) {
+    return getBucketNumber(getBucketHashCodeOld(bucketFields, bucketFieldInspectors), totalBuckets);
+  }
+
+  /**
+   * https://cwiki.apache.org/confluence/display/Hive/LanguageManual+DDL+BucketedTables
+   * @param hashCode as produced by {@link #getBucketHashCode(Object[], ObjectInspector[])}
+   */
+  public static int getBucketNumber(int hashCode, int numberOfBuckets) {
+    if(numberOfBuckets <= 0) {
+      //note that (X % 0) is illegal and (X % -1) = 0
+      // -1 is a common default when the value is missing
+      throw new IllegalArgumentException("Number of Buckets must be > 0");
+    }
+    return (hashCode & Integer.MAX_VALUE) % numberOfBuckets;
+  }
+
+  /**
+   * Computes the hash code for the given bucketed fields
+   * @param bucketFields
+   * @param bucketFieldInspectors
+   * @return
+   */
+  @Deprecated
+  public static int getBucketHashCodeOld(Object[] bucketFields, ObjectInspector[] bucketFieldInspectors) {
+    int hashCode = 0;
+    for (int i = 0; i < bucketFields.length; i++) {
+      int fieldHash = ObjectInspectorUtils.hashCode(bucketFields[i], bucketFieldInspectors[i]);
+      hashCode = 31 * hashCode + fieldHash;
+    }
+    return hashCode;
+  }
+
   public static int hashCode(Object o, ObjectInspector objIns) {
     if (o == null) {
       return 0;
@@ -545,9 +731,14 @@ public final class ObjectInspectorUtils {
       case DATE:
         return ((DateObjectInspector) poi).getPrimitiveWritableObject(o).hashCode();
       case TIMESTAMP:
-        TimestampWritable t = ((TimestampObjectInspector) poi)
-            .getPrimitiveWritableObject(o);
-        return t.hashCode();
+        // Use old timestamp writable hash code for backwards compatibility
+        TimestampWritable ts = new TimestampWritable(
+            java.sql.Timestamp.valueOf(
+                ((TimestampObjectInspector) poi).getPrimitiveWritableObject(o).toString()));
+        return ts.hashCode();
+      case TIMESTAMPLOCALTZ:
+        TimestampLocalTZWritable tstz = ((TimestampLocalTZObjectInspector) poi).getPrimitiveWritableObject(o);
+        return tstz.hashCode();
       case INTERVAL_YEAR_MONTH:
         HiveIntervalYearMonthWritable intervalYearMonth = ((HiveIntervalYearMonthObjectInspector) poi)
             .getPrimitiveWritableObject(o);
@@ -557,6 +748,8 @@ public final class ObjectInspectorUtils {
             .getPrimitiveWritableObject(o);
         return intervalDayTime.hashCode();
       case DECIMAL:
+        // Since getBucketHashCode uses this, HiveDecimal return the old (much slower) but
+        // compatible hash code.
         return ((HiveDecimalObjectInspector) poi).getPrimitiveWritableObject(o).hashCode();
 
       default: {
@@ -606,6 +799,144 @@ public final class ObjectInspectorUtils {
     }
   }
 
+  public static int getBucketHashCode(Object[] bucketFields, ObjectInspector[] bucketFieldInspectors) {
+    int hashCode = 0;
+    ByteBuffer b = ByteBuffer.allocate(8); // To be used with primitive types
+    for (int i = 0; i < bucketFields.length; i++) {
+      int fieldHash = ObjectInspectorUtils.hashCodeMurmur(
+          bucketFields[i], bucketFieldInspectors[i], b);
+      hashCode = 31 * hashCode + fieldHash;
+    }
+    return hashCode;
+  }
+
+  public static int hashCodeMurmur(Object o, ObjectInspector objIns, ByteBuffer byteBuffer) {
+    if (o == null) {
+      return 0;
+    }
+    // Reset the bytebuffer
+    byteBuffer.clear();
+    switch (objIns.getCategory()) {
+      case PRIMITIVE: {
+        PrimitiveObjectInspector poi = ((PrimitiveObjectInspector) objIns);
+        switch (poi.getPrimitiveCategory()) {
+          case VOID:
+            return 0;
+          case BOOLEAN:
+            return (((BooleanObjectInspector) poi).get(o) ? 1 : 0);
+          case BYTE:
+            return ((ByteObjectInspector) poi).get(o);
+          case SHORT: {
+            byteBuffer.putShort(((ShortObjectInspector) poi).get(o));
+            return Murmur3.hash32(byteBuffer.array(), 2);
+          }
+          case INT: {
+            byteBuffer.putInt(((IntObjectInspector) poi).get(o));
+            return Murmur3.hash32(byteBuffer.array(), 4);
+          }
+          case LONG: {
+            byteBuffer.putLong(((LongObjectInspector) poi).get(o));
+            return Murmur3.hash32(byteBuffer.array(), 8);
+          }
+          case FLOAT: {
+            byteBuffer.putFloat(Float.floatToIntBits(((FloatObjectInspector) poi).get(o)));
+            return Murmur3.hash32(byteBuffer.array(), 4);
+          }
+          case DOUBLE: {
+            // This hash function returns the same result as Double.hashCode()
+            // while DoubleWritable.hashCode returns a different result.
+            byteBuffer.putDouble(Double.doubleToLongBits(((DoubleObjectInspector) poi).get(o)));
+            return Murmur3.hash32(byteBuffer.array(), 8);
+          }
+          case STRING: {
+            // This hash function returns the same result as String.hashCode() when
+            // all characters are ASCII, while Text.hashCode() always returns a
+            // different result.
+            Text text = ((StringObjectInspector) poi).getPrimitiveWritableObject(o);
+            return Murmur3.hash32(text.getBytes(), text.getLength());
+          }
+          case CHAR: {
+            Text text = ((HiveCharObjectInspector) poi).getPrimitiveWritableObject(o).getStrippedValue();
+            return Murmur3.hash32(text.getBytes(), text.getLength());
+          }
+          case VARCHAR: {
+            Text text = ((HiveVarcharObjectInspector)poi).getPrimitiveWritableObject(o).getTextValue();
+            return Murmur3.hash32(text.getBytes(), text.getLength());
+          }
+          case BINARY:
+            return Murmur3.hash32(((BinaryObjectInspector) poi).getPrimitiveWritableObject(o).getBytes());
+
+          case DATE:
+            byteBuffer.putInt(((DateObjectInspector) poi).getPrimitiveWritableObject(o).getDays());
+            return Murmur3.hash32(byteBuffer.array(), 4);
+          case TIMESTAMP: {
+            TimestampWritableV2 t = ((TimestampObjectInspector) poi)
+                    .getPrimitiveWritableObject(o);
+            return Murmur3.hash32(t.getBytes());
+          }
+          case TIMESTAMPLOCALTZ:
+            return Murmur3.hash32((((TimestampLocalTZObjectInspector) poi).getPrimitiveWritableObject(o)).getBytes());
+          case INTERVAL_YEAR_MONTH:
+            byteBuffer.putInt(((HiveIntervalYearMonthObjectInspector) poi)
+                    .getPrimitiveWritableObject(o).hashCode());
+            return Murmur3.hash32(byteBuffer.array(), 4);
+          case INTERVAL_DAY_TIME:
+            byteBuffer.putInt(((HiveIntervalDayTimeObjectInspector) poi)
+                    .getPrimitiveWritableObject(o).hashCode());
+            return Murmur3.hash32(byteBuffer.array(), 4);
+          case DECIMAL:
+            // Since getBucketHashCode uses this, HiveDecimal return the old (much slower) but
+            // compatible hash code.
+            return Murmur3.hash32(((HiveDecimalObjectInspector) poi).getPrimitiveWritableObject(o).getInternalStorage());
+
+          default: {
+            throw new RuntimeException("Unknown type: "
+                    + poi.getPrimitiveCategory());
+          }
+        }
+      }
+      case LIST: {
+        int r = 0;
+        ListObjectInspector listOI = (ListObjectInspector)objIns;
+        ObjectInspector elemOI = listOI.getListElementObjectInspector();
+        for (int ii = 0; ii < listOI.getListLength(o); ++ii) {
+          //r = 31 * r + hashCode(listOI.getListElement(o, ii), elemOI);
+          r = 31 * r + hashCodeMurmur(listOI.getListElement(o, ii), elemOI, byteBuffer);
+        }
+        return r;
+      }
+      case MAP: {
+        int r = 0;
+        MapObjectInspector mapOI = (MapObjectInspector)objIns;
+        ObjectInspector keyOI = mapOI.getMapKeyObjectInspector();
+        ObjectInspector valueOI = mapOI.getMapValueObjectInspector();
+        Map<?, ?> map = mapOI.getMap(o);
+        for (Map.Entry<?,?> entry : map.entrySet()) {
+          r += hashCodeMurmur(entry.getKey(), keyOI, byteBuffer) ^
+                  hashCode(entry.getValue(), valueOI);
+        }
+        return r;
+      }
+      case STRUCT:
+        int r = 0;
+        StructObjectInspector structOI = (StructObjectInspector)objIns;
+        List<? extends StructField> fields = structOI.getAllStructFieldRefs();
+        for (StructField field : fields) {
+          r = 31 * r + hashCodeMurmur(structOI.getStructFieldData(o, field),
+                  field.getFieldObjectInspector(), byteBuffer);
+        }
+        return r;
+
+      case UNION:
+        UnionObjectInspector uOI = (UnionObjectInspector)objIns;
+        byte tag = uOI.getTag(o);
+        return hashCodeMurmur(uOI.getField(o), uOI.getObjectInspectors().get(tag), byteBuffer);
+
+      default:
+        throw new RuntimeException("Unknown type: "+ objIns.getTypeName());
+    }
+  }
+
   /**
    * Compare two arrays of objects with their respective arrays of
    * ObjectInspectors.
@@ -625,6 +956,41 @@ public final class ObjectInspectorUtils {
     return 0;
   }
 
+  public static int compare(Object[] o1, ObjectInspector[] oi1, Object[] o2,
+      ObjectInspector[] oi2, boolean[] columnSortOrderIsDesc, NullValueOption[] nullSortOrder) {
+    assert (o1.length == oi1.length);
+    assert (o2.length == oi2.length);
+    assert (o1.length == o2.length);
+    assert (o1.length == columnSortOrderIsDesc.length);
+    assert (o1.length == nullSortOrder.length);
+
+    for (int i = 0; i < o1.length; i++) {
+      int r = compareNull(o1[i], o2[i]);
+      if (r != 0) {
+        return nullSortOrder[i] == NullValueOption.MINVALUE ? -r : r;
+      }
+
+      if (columnSortOrderIsDesc[i]) {
+        r = compare(o2[i], oi2[i], o1[i], oi1[i]);
+      } else {
+        r = compare(o1[i], oi1[i], o2[i], oi2[i]);
+      }
+      if (r != 0) {
+        return r;
+      }
+    }
+    return 0;
+  }
+
+  public static int compareNull(Object o1, Object o2) {
+    if (o1 == null) {
+      return o2 == null ? 0 : 1;
+    } else if (o2 == null) {
+      return -1;
+    }
+    return 0;
+  }
+
   /**
    * Whether comparison is supported for this type.
    * Currently all types that references any map are not comparable.
@@ -670,17 +1036,28 @@ public final class ObjectInspectorUtils {
 
   /**
    * Compare two objects with their respective ObjectInspectors.
+   * Treat null as minimum value.
    */
   public static int compare(Object o1, ObjectInspector oi1, Object o2,
       ObjectInspector oi2, MapEqualComparer mapEqualComparer) {
+    return compare(o1, oi1, o2, oi2, mapEqualComparer, NullValueOption.MINVALUE);
+  }
+
+  /**
+   * Compare two objects with their respective ObjectInspectors.
+   * if nullValueOpt is MAXVALUE, treat null as maximum value.
+   * if nullValueOpt is MINVALUE, treat null as minimum value.
+   */
+  public static int compare(Object o1, ObjectInspector oi1, Object o2,
+      ObjectInspector oi2, MapEqualComparer mapEqualComparer, NullValueOption nullValueOpt) {
     if (oi1.getCategory() != oi2.getCategory()) {
       return oi1.getCategory().compareTo(oi2.getCategory());
     }
 
     if (o1 == null) {
-      return o2 == null ? 0 : -1;
+      return o2 == null ? 0 : nullValueOpt.getCmpReturnValue();
     } else if (o2 == null) {
-      return 1;
+      return -nullValueOpt.getCmpReturnValue();
     }
 
     switch (oi1.getCategory()) {
@@ -722,12 +1099,26 @@ public final class ObjectInspectorUtils {
       case FLOAT: {
         float v1 = ((FloatObjectInspector) poi1).get(o1);
         float v2 = ((FloatObjectInspector) poi2).get(o2);
-        return Float.compare(v1, v2);
+
+        // The IEEE 754 floating point spec specifies that signed -0.0 and 0.0 should be treated as equal.
+        if (v1 == 0.0f && v2 == 0.0f) {
+          return 0;
+        } else {
+          // Float.compare() treats -0.0 and 0.0 as different
+          return Float.compare(v1, v2);
+        }
       }
       case DOUBLE: {
         double v1 = ((DoubleObjectInspector) poi1).get(o1);
         double v2 = ((DoubleObjectInspector) poi2).get(o2);
-        return Double.compare(v1, v2);
+
+        // The IEEE 754 floating point spec specifies that signed -0.0 and 0.0 should be treated as equal.
+        if (v1 == 0.0d && v2 == 0.0d) {
+          return 0;
+        } else {
+          // Double.compare() treats -0.0 and 0.0 as different
+          return Double.compare(v1, v2);
+        }
       }
       case STRING: {
         if (poi1.preferWritable() || poi2.preferWritable()) {
@@ -759,19 +1150,26 @@ public final class ObjectInspectorUtils {
       }
 
       case DATE: {
-        DateWritable d1 = ((DateObjectInspector) poi1)
+        DateWritableV2 d1 = ((DateObjectInspector) poi1)
             .getPrimitiveWritableObject(o1);
-        DateWritable d2 = ((DateObjectInspector) poi2)
+        DateWritableV2 d2 = ((DateObjectInspector) poi2)
             .getPrimitiveWritableObject(o2);
         return d1.compareTo(d2);
       }
       case TIMESTAMP: {
-        TimestampWritable t1 = ((TimestampObjectInspector) poi1)
+        TimestampWritableV2 t1 = ((TimestampObjectInspector) poi1)
             .getPrimitiveWritableObject(o1);
-        TimestampWritable t2 = ((TimestampObjectInspector) poi2)
+        TimestampWritableV2 t2 = ((TimestampObjectInspector) poi2)
             .getPrimitiveWritableObject(o2);
         return t1.compareTo(t2);
       }
+      case TIMESTAMPLOCALTZ: {
+        TimestampLocalTZWritable tstz1 = ((TimestampLocalTZObjectInspector) poi1).
+            getPrimitiveWritableObject(o1);
+        TimestampLocalTZWritable tstz2 = ((TimestampLocalTZObjectInspector) poi2).
+            getPrimitiveWritableObject(o2);
+        return tstz1.compareTo(tstz2);
+      }
       case INTERVAL_YEAR_MONTH: {
         HiveIntervalYearMonthWritable i1 = ((HiveIntervalYearMonthObjectInspector) poi1)
             .getPrimitiveWritableObject(o1);
@@ -805,11 +1203,19 @@ public final class ObjectInspectorUtils {
       List<? extends StructField> fields1 = soi1.getAllStructFieldRefs();
       List<? extends StructField> fields2 = soi2.getAllStructFieldRefs();
       int minimum = Math.min(fields1.size(), fields2.size());
+      Object data1 =
+              soi1 instanceof ConstantObjectInspector
+              ? ((ConstantObjectInspector) soi1).getWritableConstantValue()
+              : o1;
+      Object data2 =
+              soi2 instanceof ConstantObjectInspector
+              ? ((ConstantObjectInspector) soi2).getWritableConstantValue()
+              : o2;
       for (int i = 0; i < minimum; i++) {
-        int r = compare(soi1.getStructFieldData(o1, fields1.get(i)), fields1
-            .get(i).getFieldObjectInspector(), soi2.getStructFieldData(o2,
+        int r = compare(soi1.getStructFieldData(data1, fields1.get(i)), fields1
+            .get(i).getFieldObjectInspector(), soi2.getStructFieldData(data2,
             fields2.get(i)), fields2.get(i).getFieldObjectInspector(),
-            mapEqualComparer);
+            mapEqualComparer, nullValueOpt);
         if (r != 0) {
           return r;
         }
@@ -824,7 +1230,7 @@ public final class ObjectInspectorUtils {
         int r = compare(loi1.getListElement(o1, i), loi1
             .getListElementObjectInspector(), loi2.getListElement(o2, i), loi2
             .getListElementObjectInspector(),
-            mapEqualComparer);
+            mapEqualComparer, nullValueOpt);
         if (r != 0) {
           return r;
         }
@@ -849,7 +1255,7 @@ public final class ObjectInspectorUtils {
       return compare(uoi1.getField(o1),
           uoi1.getObjectInspectors().get(tag1),
           uoi2.getField(o2), uoi2.getObjectInspectors().get(tag2),
-          mapEqualComparer);
+          mapEqualComparer, nullValueOpt);
     }
     default:
       throw new RuntimeException("Compare on unknown type: "
@@ -897,7 +1303,7 @@ public final class ObjectInspectorUtils {
           ObjectInspectorOptions.JAVA);
       return oi.getTypeName();
     } catch (Throwable e) {
-      LOG.info(StringUtils.stringifyException(e));
+      LOG.info("Failed to get Type Name From Java Class", e);
       return "unknown";
     }
   }
@@ -1059,6 +1465,24 @@ public final class ObjectInspectorUtils {
               ObjectInspectorCopyOption.WRITABLE
             ),
             (Map<?, ?>)writableValue);
+      case STRUCT:
+          StructObjectInspector soi = (StructObjectInspector) oi;
+          List<? extends StructField> fields = soi.getAllStructFieldRefs();
+          List<String> fieldNames = new ArrayList<String>(fields.size());
+          List<ObjectInspector> fieldObjectInspectors = new ArrayList<ObjectInspector>(
+            fields.size());
+          for (StructField f : fields) {
+            fieldNames.add(f.getFieldName());
+            fieldObjectInspectors.add(getStandardObjectInspector(f
+            .getFieldObjectInspector(), ObjectInspectorCopyOption.WRITABLE));
+          }
+          if (value != null && (writableValue.getClass().isArray())) {
+            writableValue = java.util.Arrays.asList((Object[])writableValue);
+          }
+          return ObjectInspectorFactory.getStandardConstantStructObjectInspector(
+            fieldNames,
+            fieldObjectInspectors,
+            (List<?>)writableValue);
       default:
        throw new IllegalArgumentException(
            writableOI.getCategory() + " not yet supported for constant OI");
@@ -1074,6 +1498,7 @@ public final class ObjectInspectorUtils {
       case PRIMITIVE:
       case LIST:
       case MAP:
+      case STRUCT:
         return true;
       default:
         return false;
@@ -1120,6 +1545,8 @@ public final class ObjectInspectorUtils {
       return oi instanceof SettableDateObjectInspector;
     case TIMESTAMP:
       return oi instanceof SettableTimestampObjectInspector;
+    case TIMESTAMPLOCALTZ:
+      return oi instanceof SettableTimestampLocalTZObjectInspector;
     case INTERVAL_YEAR_MONTH:
       return oi instanceof SettableHiveIntervalYearMonthObjectInspector;
     case INTERVAL_DAY_TIME:
@@ -1164,8 +1591,8 @@ public final class ObjectInspectorUtils {
    *
    * @param oi - Input object inspector
    * @param oiSettableProperties - Lookup map to cache the result.(If no caching, pass null)
-   * @return - true if : (1) oi is an instance of settable<DataType>OI.
-   *                     (2) All the embedded object inspectors are instances of settable<DataType>OI.
+   * @return - true if : (1) oi is an instance of settable&lt;DataType&gt;OI.
+   *                     (2) All the embedded object inspectors are instances of settable&lt;DataType&gt;OI.
    *           If (1) or (2) is false, return false.
    */
   public static boolean hasAllFieldsSettable(ObjectInspector oi,
