@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,6 +19,7 @@
 package org.apache.hive.service.cli;
 
 import java.io.IOException;
+import java.nio.ByteBuffer;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.CancellationException;
@@ -43,6 +44,7 @@ import org.apache.hive.service.ServiceUtils;
 import org.apache.hive.service.auth.HiveAuthFactory;
 import org.apache.hive.service.cli.operation.Operation;
 import org.apache.hive.service.cli.session.SessionManager;
+import org.apache.hive.service.rpc.thrift.TOperationHandle;
 import org.apache.hive.service.rpc.thrift.TProtocolVersion;
 import org.apache.hive.service.server.HiveServer2;
 import org.slf4j.Logger;
@@ -71,16 +73,19 @@ public class CLIService extends CompositeService implements ICLIService {
   // The HiveServer2 instance running this service
   private final HiveServer2 hiveServer2;
   private int defaultFetchRows;
+  // This is necessary for tests and embedded mode, where HS2 init is not executed.
+  private boolean allowSessionsInitial;
 
-  public CLIService(HiveServer2 hiveServer2) {
+  public CLIService(HiveServer2 hiveServer2, boolean allowSessions) {
     super(CLIService.class.getSimpleName());
     this.hiveServer2 = hiveServer2;
+    this.allowSessionsInitial = allowSessions;
   }
 
   @Override
   public synchronized void init(HiveConf hiveConf) {
-    this.hiveConf = hiveConf;
-    sessionManager = new SessionManager(hiveServer2);
+    setHiveConf(hiveConf);
+    sessionManager = new SessionManager(hiveServer2, allowSessionsInitial);
     defaultFetchRows = hiveConf.getIntVar(ConfVars.HIVE_SERVER2_THRIFT_RESULTSET_DEFAULT_FETCH_SIZE);
     addService(sessionManager);
     //  If the hadoop cluster is secure, do a kerberos login for the service from the keytab
@@ -131,6 +136,7 @@ public class CLIService extends CompositeService implements ICLIService {
   }
 
   private void setupBlockedUdfs() {
+    HiveConf hiveConf = getHiveConf();
     FunctionRegistry.setupPermissionsForBuiltinUDFs(
         hiveConf.getVar(ConfVars.HIVE_SERVER2_BUILTIN_UDF_WHITELIST),
         hiveConf.getVar(ConfVars.HIVE_SERVER2_BUILTIN_UDF_BLACKLIST));
@@ -430,6 +436,33 @@ public class CLIService extends CompositeService implements ICLIService {
     return opHandle;
   }
 
+  /* (non-Javadoc)
+   * @see org.apache.hive.service.cli.ICLIService#uploadData(org.apache.hive.service.cli.SessionHandle)
+   */
+  @Override
+  public OperationHandle uploadData(
+      SessionHandle sessionHandle,
+      ByteBuffer values,
+      String tableName,
+      String path) throws HiveSQLException {
+    LOG.info(sessionHandle + ": uploadData()");
+    return sessionManager.getSession(sessionHandle).uploadData(values, tableName, path);
+  }
+
+  /* (non-Javadoc)
+   * @see org.apache.hive.service.cli.ICLIService#downloadData(org.apache.hive.service.cli.SessionHandle)
+   */
+  @Override
+  public OperationHandle downloadData(
+      SessionHandle sessionHandle,
+      String tableName,
+      String query,
+      String format,
+      Map<String, String> options) throws HiveSQLException {
+    LOG.info(sessionHandle + ": downloadData()");
+    return sessionManager.getSession(sessionHandle).downloadData(tableName, query, format, options);
+  }
+
   /* (non-Javadoc)
    * @see org.apache.hive.service.cli.ICLIService#getOperationStatus(org.apache.hive.service.cli.OperationHandle)
    */
@@ -448,7 +481,7 @@ public class CLIService extends CompositeService implements ICLIService {
           HiveConf.ConfVars.HIVE_SERVER2_LONG_POLLING_TIMEOUT, TimeUnit.MILLISECONDS);
 
       final long elapsed = System.currentTimeMillis() - operation.getBeginTime();
-      // A step function to increase the polling timeout by 500 ms every 10 sec, 
+      // A step function to increase the polling timeout by 500 ms every 10 sec,
       // starting from 500 ms up to HIVE_SERVER2_LONG_POLLING_TIMEOUT
       final long timeout = Math.min(maxTimeout, (elapsed / TimeUnit.SECONDS.toMillis(10) + 1) * 500);
 
@@ -472,9 +505,15 @@ public class CLIService extends CompositeService implements ICLIService {
     }
     OperationStatus opStatus = operation.getStatus();
     LOG.debug(opHandle + ": getOperationStatus()");
+    long numModifiedRows = operation.getNumModifiedRows();
+    opStatus.setNumModifiedRows(numModifiedRows);
     opStatus.setJobProgressUpdate(progressUpdateLog(getProgressUpdate, operation, conf));
     return opStatus;
   }
+  public HiveConf getHiveSessionConf(OperationHandle opHandle) throws HiveSQLException {
+    Operation operation = sessionManager.getOperationManager().getOperation(opHandle);
+    return operation.getParentSession().getHiveConf();
+  }
 
   public HiveConf getSessionConf(SessionHandle sessionHandle)
       throws HiveSQLException {
@@ -487,18 +526,19 @@ public class CLIService extends CompositeService implements ICLIService {
         || !OperationType.EXECUTE_STATEMENT.equals(operation.getType())) {
       return new JobProgressUpdate(ProgressMonitor.NULL);
     }
-    
+
     SessionState sessionState = operation.getParentSession().getSessionState();
     long startTime = System.nanoTime();
     int timeOutMs = 8;
+    boolean terminated = operation.isDone();
     try {
-      while (sessionState.getProgressMonitor() == null && !operation.isDone()) {
+      while ((sessionState.getProgressMonitor() == null) && !terminated) {
         long remainingMs = (PROGRESS_MAX_WAIT_NS - (System.nanoTime() - startTime)) / 1000000l;
         if (remainingMs <= 0) {
           LOG.debug("timed out and hence returning progress log as NULL");
           return new JobProgressUpdate(ProgressMonitor.NULL);
         }
-        Thread.sleep(Math.min(remainingMs, timeOutMs));
+        terminated = operation.waitToTerminate(Math.min(remainingMs, timeOutMs));
         timeOutMs <<= 1;
       }
     } catch (InterruptedException e) {
@@ -562,8 +602,9 @@ public class CLIService extends CompositeService implements ICLIService {
   }
 
   // obtain delegation token for the give user from metastore
-  public synchronized String getDelegationTokenFromMetaStore(String owner)
+  public String getDelegationTokenFromMetaStore(String owner)
       throws HiveSQLException, UnsupportedOperationException, LoginException, IOException {
+    HiveConf hiveConf = getHiveConf();
     if (!hiveConf.getBoolVar(HiveConf.ConfVars.METASTORE_USE_THRIFT_SASL) ||
         !hiveConf.getBoolVar(HiveConf.ConfVars.HIVE_SERVER2_ENABLE_DOAS)) {
       throw new UnsupportedOperationException(
@@ -591,6 +632,11 @@ public class CLIService extends CompositeService implements ICLIService {
     return delegationToken;
   }
 
+  @Override
+  public void setApplicationName(SessionHandle sh, String value) throws HiveSQLException {
+    sessionManager.getSession(sh).setApplicationName(value);
+  }
+
   @Override
   public void cancelDelegationToken(SessionHandle sessionHandle, HiveAuthFactory authFactory,
       String tokenStr) throws HiveSQLException {
@@ -606,6 +652,15 @@ public class CLIService extends CompositeService implements ICLIService {
     LOG.info(sessionHandle  + ": renewDelegationToken()");
   }
 
+  @Override
+  public String getQueryId(TOperationHandle opHandle) throws HiveSQLException {
+    Operation operation = sessionManager.getOperationManager().getOperation(
+        new OperationHandle(opHandle));
+    final String queryId = operation.getQueryId();
+    LOG.debug(opHandle + ": getQueryId() " + queryId);
+    return queryId;
+  }
+
   public SessionManager getSessionManager() {
     return sessionManager;
   }
