@@ -39,10 +39,11 @@ import org.apache.hadoop.hive.cli.CliSessionState;
 import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.hive.metastore.HiveMetaStoreClient;
 import org.apache.hadoop.hive.metastore.IMetaStoreClient;
-import org.apache.hadoop.hive.metastore.txn.TxnDbUtil;
+import org.apache.hadoop.hive.metastore.conf.MetastoreConf;
+import org.apache.hadoop.hive.metastore.utils.TestTxnDbUtil;
 import org.apache.hadoop.hive.ql.DriverFactory;
 import org.apache.hadoop.hive.ql.IDriver;
-import org.apache.hadoop.hive.ql.processors.CommandProcessorResponse;
+import org.apache.hadoop.hive.ql.processors.CommandProcessorException;
 import org.apache.hadoop.hive.ql.session.SessionState;
 import org.apache.hadoop.hive.serde.serdeConstants;
 import org.apache.thrift.TException;
@@ -130,16 +131,16 @@ public class TestStreamingDynamicPartitioning {
     conf
       .setVar(HiveConf.ConfVars.HIVE_AUTHORIZATION_MANAGER,
         "org.apache.hadoop.hive.ql.security.authorization.plugin.sqlstd.SQLStdHiveAuthorizerFactory");
-    TxnDbUtil.setConfValues(conf);
+    TestTxnDbUtil.setConfValues(conf);
     conf.setBoolVar(HiveConf.ConfVars.METASTORE_EXECUTE_SET_UGI, true);
     conf.setBoolVar(HiveConf.ConfVars.HIVE_SUPPORT_CONCURRENCY, true);
-    conf.setVar(HiveConf.ConfVars.DYNAMICPARTITIONINGMODE, "nonstrict");
     dbFolder.create();
+    MetastoreConf.setVar(conf, MetastoreConf.ConfVars.WAREHOUSE, "raw://" + dbFolder.newFolder("warehouse"));
     loc1 = dbFolder.newFolder(dbName + ".db").toString();
 
     //1) Start from a clean slate (metastore)
-    TxnDbUtil.cleanDb(conf);
-    TxnDbUtil.prepDb(conf);
+    TestTxnDbUtil.cleanDb(conf);
+    TestTxnDbUtil.prepDb(conf);
 
     //2) obtain metastore clients
     msClient = new HiveMetaStoreClient(conf);
@@ -512,7 +513,7 @@ public class TestStreamingDynamicPartitioning {
     assertEquals("7\tfoo\t" + defaultPartitionName + "\t" + defaultPartitionName, res.get(6));
     assertEquals("8\tbar\t" + defaultPartitionName + "\t12", res.get(7));
 
-    defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULTPARTITIONNAME);
+    defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULT_PARTITION_NAME);
     res = queryTable(driver, "show partitions " + (dbName + "." + tblName));
     assertEquals(5, res.size());
     assertTrue(res.contains("year=2018/month=2"));
@@ -572,7 +573,7 @@ public class TestStreamingDynamicPartitioning {
     assertTrue(exception.getMessage().equals("Transaction state is not OPEN. Missing beginTransaction?"));
 
     connection.close();
-    String defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULTPARTITIONNAME);
+    String defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULT_PARTITION_NAME);
     List<String> res = queryTable(driver, "select * from " + (dbName + "." + tblName) + " order by id");
     assertEquals(2, res.size());
     assertEquals("1\tfoo\tAsia\t" + defaultPartitionName, res.get(0));
@@ -706,7 +707,7 @@ public class TestStreamingDynamicPartitioning {
     assertNotNull(exception);
     assertTrue(exception.getMessage().endsWith("Streaming connection is closed already."));
 
-    String defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULTPARTITIONNAME);
+    String defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULT_PARTITION_NAME);
     List<String> res = queryTable(driver, "select * from " + (dbName + "." + tblName) + " order by id");
     assertEquals(2, res.size());
     assertEquals("1\tfoo\tAsia\t" + defaultPartitionName, res.get(0));
@@ -757,7 +758,7 @@ public class TestStreamingDynamicPartitioning {
     }
     assertNotNull(exception);
     assertTrue(exception.getMessage().equals("Streaming connection is closed already."));
-    String defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULTPARTITIONNAME);
+    String defaultPartitionName = conf.getVar(HiveConf.ConfVars.DEFAULT_PARTITION_NAME);
     List<String> res = queryTable(driver, "select * from " + (dbName + "." + tblName) + " order by id");
     assertEquals(3, res.size());
     assertEquals("1\tfoo\tAsia\t" + defaultPartitionName, res.get(0));
@@ -826,19 +827,21 @@ public class TestStreamingDynamicPartitioning {
   private static boolean runDDL(IDriver driver, String sql) {
     LOG.debug(sql);
     System.out.println(sql);
-    CommandProcessorResponse cpr = driver.run(sql);
-    if (cpr.getResponseCode() == 0) {
+    try {
+      driver.run(sql);
       return true;
+    } catch (CommandProcessorException e) {
+      LOG.error("Statement: " + sql + " failed: " + e);
+      return false;
     }
-    LOG.error("Statement: " + sql + " failed: " + cpr);
-    return false;
   }
 
 
   private static ArrayList<String> queryTable(IDriver driver, String query) throws IOException {
-    CommandProcessorResponse cpr = driver.run(query);
-    if (cpr.getResponseCode() != 0) {
-      throw new RuntimeException(query + " failed: " + cpr);
+    try {
+      driver.run(query);
+    } catch (CommandProcessorException e) {
+      throw new RuntimeException(query + " failed: " + e);
     }
     ArrayList<String> res = new ArrayList<String>();
     driver.getResults(res);
